## the following packages are required
library(plyr)
library(glasso)
library(Rcpp)
library(RcppArmadillo)
library(glmnet)
library(penalizedLDA)
library(MASS)
library(sparseLDA)

dat = readRDS("eegdat.RDS")

# ----- matrices for selecting tuning parameters based on 5-fold CV
lambda.lasso = 2^seq(-9, -1, length=100)
CV.lasso.mat = 0
CV.lasso.inner = matrix(0, nrow=length(lambda.lasso), ncol=5)
Oracle.lasso.mat = rep(0, length(lambda.lasso))

lambda.Witten = 2^seq(-14, 4, length=100)
CV.Witten.mat = 0
CV.Witten.inner = matrix(0, nrow=length(lambda.lasso), ncol=5)
Oracle.Witten.mat = rep(0, length(lambda.lasso))

X = dat$EEG.avg[,,-uu]
r = dim(X)[1]
class = dat$class[-uu]+1 # class codes where 0/1

r = dim(X)[1]
p = dim(X)[2]

X.test = array(dat$EEG.avg[,,uu], dim=c(r,p,1))
class.test = dat$class[uu]+1

# ----- randomly split training data within each class
set.seed(seed)	# seed = 92489 + uu
class1.set = which(class==1)
fold.class1 = sample(rep(1:5, length=length(class1.set)))
test.inds.class1 = split(class1.set, fold.class1)

class2.set = which(class==2)
fold.class2 = sample(rep(1:5, length=length(class2.set)))
test.inds.class2 = split(class2.set, fold.class2)

# ----- merge classes
test.inds = mapply(c, test.inds.class1, test.inds.class2, SIMPLIFY=FALSE)

# ---- perform 5-fold CV
for(hhh in 1:5){

	X.train = X[,,-test.inds[[hhh]]]
	class.train = class[-test.inds[[hhh]]]

	X.innertest = X[,,test.inds[[hhh]]]
	vecX.innertest = apply(X.innertest, 3, as.vector)
	class.innertest = class[test.inds[[hhh]]]
	
	# ---  glmnet
	out = glmnet(x=t(apply(X.train, 3, as.vector)),y=as.factor(class.train), family="binomial", lambda=lambda.lasso)
	CV.lasso.inner[,hhh] = colSums(predict(out, newx=t(vecX.innertest), type="class") != class.innertest)/length(class.innertest)

	# --- Witten
	for(j in 1:length(lambda.Witten)){
		CV.Witten.inner[j,hhh]= sum(PenalizedLDA(x=t(apply(X.train, 3, as.vector)),y=class.train,lambda=lambda.Witten[j],K=1, xte=t(apply(X.innertest, 3, as.vector)))$ypred[,1] != class.innertest)/length(class.innertest)
	}

}
	

for(ggg in 1:5){

	CV.lasso.mat = CV.lasso.mat + (length(test.inds[[ggg]])/dim(X)[3])*CV.lasso.inner[,ggg]
	CV.Witten.mat = CV.Witten.mat + (length(test.inds[[ggg]])/dim(X)[3])*CV.Witten.inner[,ggg]

}

out = glmnet(x=t(apply(X, 3, as.vector)),y=as.factor(class), family="binomial", lambda=lambda.lasso)
Oracle.lasso.mat = (colSums(predict(out, newx=t(apply(X.test, 3, as.vector)), type="class") != class.test)/length(class.test))

for(j in 1:length(lambda.Witten)){
	Oracle.Witten.mat[j]= sum(PenalizedLDA(x=t(apply(X, 3, as.vector)),y=class,lambda=lambda.Witten[j],K=1, xte=t(apply(X.test, 3, as.vector)))$ypred[,1] != class.test)/length(class.test)
}



results = list(
	"CV.lasso.mat" = CV.lasso.mat, 
	"CV.lasso.inner" = CV.lasso.inner, 
	"Oracle.lasso.mat" = Oracle.lasso.mat, 

	"CV.Witten.mat" =CV.Witten.mat, 
	"CV.Witten.inner" = CV.Witten.inner,
	"Oracle.Witten.mat" = Oracle.Witten.mat)


saveRDS(results, file=savename)



