library(plyr)
library(glasso)
library(Rcpp)
library(RcppArmadillo)
library(glmnet)
library(penalizedLDA)
library(klaR)
library(msda)
library(MASS)
library(sparseLDA)

dat = readRDS("handshapes.RDS")

lambda.lasso = 2^seq(-10, 0, length=100)
CV.lasso.mat = rep(0, length(lambda.lasso))
CV.lasso.inner = matrix(0,nrow=length(lambda.lasso), ncol= 5)
Oracle.lasso.mat = rep(0, length(lambda.lasso))

lambda.msda = 2^seq(-1, -4, length=100)
CV.msda.mat = rep(0, length(lambda.msda))
CV.msda.inner = matrix(0, nrow=length(lambda.msda), ncol= 5)
Oracle.msda.mat = rep(0, length(lambda.msda))

lambda.Witten = 2^seq(-10, -4, length=100)
CV.Witten.mat = rep(0, length(lambda.Witten))
CV.Witten.inner = matrix(0, nrow=length(lambda.Witten), ncol= 5)
Oracle.Witten.mat = rep(0, length(lambda.Witten))


# --- omit uuth data point.
X = dat$X[,,-uu]

class = as.factor(dat$class)
class.test = class[uu]
class = class[-uu]

r = dim(X)[1]
p = dim(X)[2]

X.test = array(dat$X[,,uu], dim=c(r,p,1))

# ----- randomly split training data within each class
set.seed(seed) # seed = uu 
	
class1.set = which(class==1)
fold.class1 = sample(rep(1:5, length=length(class1.set)))
test.inds.class1 = split(class1.set, fold.class1, )

class2.set = which(class==2)
fold.class2 = sample(rep(1:5, length=length(class2.set)))
test.inds.class2 = split(class2.set, fold.class2)

class3.set = which(class==3)
fold.class3 = sample(rep(1:5, length=length(class3.set)))
test.inds.class3 = split(class3.set, fold.class3)

class4.set = which(class==4)
fold.class4 = sample(rep(1:5, length=length(class4.set)))
test.inds.class4 = split(class4.set, fold.class4)

# ----- merge splits
test.inds = mapply(c, test.inds.class1, test.inds.class2, 
	test.inds.class3, test.inds.class4, SIMPLIFY=FALSE)

# ------ perform 5-fold CV
for(hhh in 1:5){

	X.train = X[,,-test.inds[[hhh]]]
	class.train = class[-test.inds[[hhh]]]

	X.innertest = X[,,test.inds[[hhh]]]
	vecX.innertest = apply(X.innertest, 3, as.vector)
	class.innertest = class[test.inds[[hhh]]]
	
	# ---- glmnet
	out = glmnet(x=t(apply(X.train, 3, as.vector)),y=class.train, family="multinomial", lambda=lambda.lasso)
	CV.lasso.inner[,hhh] = colSums(predict(out, newx=t(vecX.innertest), type="class") != class.innertest)/length(class.innertest)

	# ---- Witten
	for(j in 1:length(lambda.Witten)){
		CV.Witten.inner[j,hhh]= sum(PenalizedLDA(x=t(apply(X.train, 3, as.vector)),y=class.train,lambda=lambda.Witten[j],K=1, xte=t(apply(X.innertest, 3, as.vector)))$ypred[,1] != class.innertest)/length(class.innertest)
	}

	# ------- MSDA 
	out = msda(x=t(apply(X.train, 3, as.vector)), y=class.train, lambda=lambda.msda)
	store = (colSums(predict(out, newx=t(apply(X.innertest, 3, as.vector)), type="class") != class.innertest)/length(class.innertest))
	CV.msda.inner[1:length(store),hhh] = store
	
	# ---- replace missing with random guess 
	CV.msda.inner[(length(store)+1):length(lambda.msda),hhh] = sum(sample(1:4, length(class.innertest), prob = c(count(class.train)[,2]/length(class.train)), replace=TRUE) != class.innertest)/length(class.innertest)

}

# --- aggregate over folds 
for(ggg in 1:5){

	CV.lasso.mat = CV.lasso.mat + (length(test.inds[[ggg]])/dim(X)[3])*CV.lasso.inner[,ggg]
	CV.Witten.mat = CV.Witten.mat + (length(test.inds[[ggg]])/dim(X)[3])*CV.Witten.inner[,ggg]
	CV.msda.mat = CV.msda.mat + (length(test.inds[[ggg]])/dim(X)[3])*CV.msda.inner[,ggg]

}

# ---- fit entire grid 
out = glmnet(x=t(apply(X, 3, as.vector)),y=class, family="multinomial", lambda=lambda.lasso)
Oracle.lasso.mat = 1*(predict(out, newx=t(apply(X.test, 3, as.vector)), type="class") != class.test)

for(j in 1:length(lambda.Witten)){
	Oracle.Witten.mat[j]= sum(PenalizedLDA(x=t(apply(X, 3, as.vector)),y=class,lambda=lambda.Witten[j],K=1, xte=t(apply(X.test, 3, as.vector)))$ypred[,1] != class.test)/length(class.test)
}

out = msda(x=t(apply(X, 3, as.vector)), y=class, lambda=lambda.msda)
store = 1*(predict(out, newx=t(apply(X.test, 3, as.vector)), type="class") != class.test)
Oracle.msda.mat[1:length(store)] = store
Oracle.msda.mat[(length(store)+1):length(lambda.msda)] = 1*(sample(1:4, length(class.test), prob = c(count(class)[,2]/length(class)), replace=TRUE) != class.test)


results = list(
	"CV.lasso.mat" = CV.lasso.mat, 
	"CV.lasso.inner" = CV.lasso.inner, 
	"Oracle.lasso.mat" = Oracle.lasso.mat, 

	"CV.msda.mat" = CV.msda.mat, 
	"CV.msda.inner" = CV.msda.inner, 
	"Oracle.msda.mat" = Oracle.msda.mat, 

	"CV.Witten.mat" =CV.Witten.mat, 
	"CV.Witten.inner" = CV.Witten.inner,
	"Oracle.Witten.mat" = Oracle.Witten.mat)


saveRDS(results, file=savename)



