library(plyr)
library(glasso)
library(Rcpp)
library(RcppArmadillo)

dat = readRDS("handshapes.RDS")
sourceCpp("Functions/M_Update_Rcpp.cpp") 
source("Functions/SAMPLES.R") 
source("Functions/GENERAL_ITER.R") 
source("Functions/MN_CLASSIFICATION.R") 
source("Functions/MatLDA.R")
source("Functions/MatLDA_Grid.R") 
source("Functions/MN_MLE.R") 


lambda2 = c(2^seq(-9, -5, by=.5))
lambda1 = c(2^seq(-11, -5, by=.5))

# --- matrices for tuning parameter selection 
Oracle.mat = matrix(0, nrow=length(lambda1), ncol=length(lambda2))
CV.mat = matrix(0, nrow=length(lambda1), ncol=length(lambda2))
CV.inner = array(0, dim=c(length(lambda1), length(lambda2), 5))


# --- omit uuth data point.
X = dat$X[,,-uu]

class = dat$class[-uu]

r = dim(X)[1]
p = dim(X)[2]
C = 4

X.test = array(dat$X[,,uu], dim=c(r,p,1))
class.test = dat$class[uu]

#---- randomly split training data within each class
set.seed(seed)	# seed = 66923 + uu

class1.set = which(class==1)
fold.class1 = sample(rep(1:5, length=length(class1.set)))
test.inds.class1 = split(class1.set, fold.class1)

class2.set = which(class==2)
fold.class2 = sample(rep(1:5, length=length(class2.set)))
test.inds.class2 = split(class2.set, fold.class2)

class3.set = which(class==3)
fold.class3 = sample(rep(1:5, length=length(class3.set)))
test.inds.class3 = split(class3.set, fold.class3)

class4.set = which(class==4)
fold.class4 = sample(rep(1:5, length=length(class4.set)))
test.inds.class4 = split(class4.set, fold.class4)

#---- merge classes
test.inds = mapply(c, test.inds.class1, test.inds.class2,
test.inds.class3, test.inds.class4, SIMPLIFY=FALSE)

#---- redo for each; inner
Val.inner = array(0, dim=c(length(lambda1), length(lambda2), 5))

#---- perform 5-fold CV
for(hhh in 1:5){

	X.train = X[,,-test.inds[[hhh]]]
	class.train = class[-test.inds[[hhh]]]

	X.innertest = X[,,test.inds[[hhh]]]
	class.innertest = class[test.inds[[hhh]]]

	# ----- go through entire grid
	fit.grid = MatLDA_Grid(X=X.train, class=class.train, lambda1=lambda1, 
	lambda2=lambda2, quiet=TRUE,
	Xval=X.innertest, classval= class.innertest,
	k.iter = 100, cov.tol=1e-5, m.tol=1e-5, full.tol=1e-6)

	CV.inner[,, hhh] = fit.grid$Val.mat
	cat("Finished CV loop", hhh, "within omitted uu=", uu, "\n")

}

#--- aggregate over folds
for(ggg in 1:5){
	CV.mat = CV.mat + (length(test.inds[[ggg]])/dim(X)[3])*CV.inner[,, ggg]
}

# ----- fit entire grid for oracle mat
fit.grid =  MatLDA_Grid(X=X, class=class, lambda1=lambda1, 
lambda2=lambda2, quiet=FALSE,
Xval=X.test, classval= class.test,
k.iter = 100, cov.tol=1e-5, m.tol=1e-5, full.tol=1e-6)

G.mat = fit.grid$G.mat
Oracle.mat = fit.grid$Val.mat

cat("through omitted ", uu, "\n")

results = list(
	"CV.mat"=CV.mat, 
	"G.mat"=G.mat,
	"CV.array"=CV.inner, 
	"Oracle.mat" = Oracle.mat)


saveRDS(results, file=savename)


