# -----------------------------------------------
# Functions for TPR/TNR tables 
# -----------------------------------------------
verticalize = function(x, y){
	g = length(x)
	out.vec = matrix(0, nrow=g, ncol=1)
	for(j in 1:g){
		out.vec[j,] = paste(round(signif(x[j], 3), 3), paste("footnotesize(", round(signif(y[j], 3), 2), ")"))
	}
	return(out.vec)
}

collateTPR_TNR = function(x, y){
	g = length(x[1,])
	out.vec = matrix(0, nrow=g, ncol=1)
	for(j in 1:g){
		out.vec[j,] = paste(signif(x[1,j], 3),"/", signif(y[1,j], 3))
	}
	return(out.vec)
}



# -----------------------------------------------
#
# r = c results 
#
# -----------------------------------------------


library(xtable)
oracle = NULL
MNLDA = NULL
propval = NULL
witten = NULL
sda = NULL
msda = NULL

temp = readRDS("Model1_r8p8_Fisher.RDS")
oracle = c(oracle, mean(temp$oracle.mc))
MNLDA =c(MNLDA, mean(temp$MNlda.mc))
propval = c(propval, mean(temp$propval.mc))
witten = c(witten, mean(temp$witten.mc))
sda = c(sda, mean(temp$sda.mc))
msda = c(msda, mean(temp$msda.mc))
temp = NULL

temp = readRDS("Model1_r16p16_Fisher.RDS")
oracle = c(oracle, mean(temp$oracle.mc))
MNLDA =c(MNLDA, mean(temp$MNlda.mc))
propval = c(propval, mean(temp$propval.mc))
witten = c(witten, mean(temp$witten.mc))
sda = c(sda, mean(temp$sda.mc))
msda = c(msda, mean(temp$msda.mc))
temp = NULL

temp = readRDS("Model1_r32p32_Fisher.RDS")
oracle = c(oracle, mean(temp$oracle.mc))
MNLDA =c(MNLDA, mean(temp$MNlda.mc))
propval = c(propval, mean(temp$propval.mc))
witten = c(witten, mean(temp$witten.mc))
sda = c(sda, mean(temp$sda.mc))
msda = c(msda, mean(temp$msda.mc))
temp = NULL


# --------------------------------------
temp.tot = list(NA)
for(k in 1:100){
	filename = paste("Model1_r64p64_Fisher_", k, ".RDS", sep="")
	temp = readRDS(filename)
	temp.tot[[k]] = temp
}
temp.tot = matrix(unlist(temp.tot), nrow=length(names(temp)), ncol=100, byrow=FALSE)
temp.tot = as.data.frame(t(temp.tot))
names(temp.tot) = names(temp)
temp = temp.tot

oracle = c(oracle, mean(temp$oracle.mc))
MNLDA =c(MNLDA, mean(temp$MNlda.mc))
propval = c(propval, mean(temp$propval.mc))
witten = c(witten, mean(temp$witten.mc))
sda = c(sda, mean(temp$sda.mc))
msda = c(msda, mean(temp$msda.mc))
temp = NULL



pdf("Model1_Square_Fisher.pdf", width=4, height=3.5)
par(xpd=T, mar=c(4.1,4.1,1.6,1.1))
plot(c(1, 2, 3, 4), oracle, ylab = "Average misclassification rate", xlab=expression("(r,c)"),  type="b", lty=1, pch=8, ylim=c(0, .40), axes=FALSE)
axis(1, at=c(1, 2, 3, 4), labels=c("(8,8)", "(16,16)", "(32,32)", "(64,64)"))
axis(2, at=seq(0, .4, by=.1))
lines(c(1, 2, 3, 4), MNLDA, lty=1, pch=16, type="b")
lines(c(1, 2, 3, 4),  witten, lty=4, pch=7, type="b")
lines(c(1, 2, 3, 4), sda, lty=4, pch=10, type="b")
lines(c(1, 2, 3, 4), msda, lty=4, pch=9, type="b")
lines(c(1, 2, 3, 4), propval, lty=1, pch=15,type="b")
dev.off()









# --------------------------------------------------
#
# r = 32 and c varying results
#
# --------------------------------------------------

library(xtable)
oracle = NULL
MNLDA = NULL
propval = NULL
witten = NULL
sda = NULL
msda = NULL


temp = readRDS("Model1_r32p8_Fisher.RDS")

oracle = c(oracle, mean(temp$oracle.mc))
MNLDA =c(MNLDA, mean(temp$MNlda.mc))
propval = c(propval, mean(temp$propval.mc))
witten = c(witten, mean(temp$witten.mc))
sda = c(sda, mean(temp$sda.mc))
msda = c(msda, mean(temp$msda.mc))
temp = NULL

temp = readRDS("Model1_r32p16_Fisher.RDS")
oracle = c(oracle, mean(temp$oracle.mc))
MNLDA =c(MNLDA, mean(temp$MNlda.mc))
propval = c(propval, mean(temp$propval.mc))
witten = c(witten, mean(temp$witten.mc))
sda = c(sda, mean(temp$sda.mc))
msda = c(msda, mean(temp$msda.mc))
temp = NULL

temp = readRDS("Model1_r32p32_Fisher.RDS")
oracle = c(oracle, mean(temp$oracle.mc))
MNLDA =c(MNLDA, mean(temp$MNlda.mc))
propval = c(propval, mean(temp$propval.mc))
witten = c(witten, mean(temp$witten.mc))
sda = c(sda, mean(temp$sda.mc))
msda = c(msda, mean(temp$msda.mc))
temp = NULL



# ---------------------------------
temp.tot = list(NA)
for(k in 1:100){
	filename = paste("Model1_r32p64_Fisher_", k, ".RDS", sep="")
	temp = readRDS(filename)
	temp.tot[[k]] = temp
}
temp.tot = matrix(unlist(temp.tot), nrow=length(names(temp)), ncol=100, byrow=FALSE)
temp.tot = as.data.frame(t(temp.tot))
names(temp.tot) = names(temp)
temp = temp.tot

oracle = c(oracle, mean(temp$oracle.mc))
MNLDA =c(MNLDA, mean(temp$MNlda.mc))
propval = c(propval, mean(temp$propval.mc))
witten = c(witten, mean(temp$witten.mc))
sda = c(sda, mean(temp$sda.mc))
msda = c(msda, mean(temp$msda.mc))
temp = NULL


# ---------------------------------
temp.tot = list(NA)
for(k in 1:100){
	filename = paste("Model1_r32p126_Fisher_", k, ".RDS", sep="")
	temp = readRDS(filename)
	temp.tot[[k]] = temp
}
temp.tot = matrix(unlist(temp.tot), nrow=length(names(temp)), ncol=100, byrow=FALSE)
temp.tot = as.data.frame(t(temp.tot))
names(temp.tot) = names(temp)
temp = temp.tot

oracle = c(oracle, mean(temp$oracle.mc))
MNLDA =c(MNLDA, mean(temp$MNlda.mc))
propval = c(propval, mean(temp$propval.mc))
witten = c(witten, mean(temp$witten.mc))
sda = c(sda, mean(temp$sda.mc))
msda = c(msda, mean(temp$msda.mc))
temp = NULL


dev.new(width=4, height=3.5)
par(xpd=T, mar=c(4.1,2.1,2.1,1.1))
plot(c(1, 2, 3, 4, 5), oracle, ylab = "", xlab=expression("(r,c)"),  type="b", lty=1, pch=8, ylim=c(0, .40), axes=FALSE)
axis(1, at=c(1, 2, 3, 4, 5), labels=c("(32,8)", "(32,16)", "(32,32)", "(32,64)", "(32, 126)"))
axis(2, at=seq(0, .40, by=.1))
lines(c(1, 2, 3, 4, 5), MNLDA, lty=1, pch=16, type="b")
lines(c(1, 2, 3, 4, 5),  witten, lty=4, pch=7, type="b")
lines(c(1, 2, 3, 4, 5), sda, lty=4, pch=10, type="b")
lines(c(1, 2, 3, 4, 5), msda, lty=4, pch=9, type="b")
lines(c(1, 2, 3, 4, 5), propval, lty=1, pch=15,type="b")
legend(locator(1), cex=.85, legend=c("Bayes", "MN", "Witten", "Clemmensen", "Mai", "PMN"), 
lty = c(1, 1, 4, 4, 4, 1), pch = c(8, 16, 7, 10, 9, 15), bg="transparent",
col=c("black", "black", "black", "black", "black", "black"), ncol=2, box.lwd = 0,box.col = "white")


