# --------------------------------------
# Create data for Model 1
# --------------------------------------


CREATE_DAT_COV1 = function(r, p, C, rho.U, rho.V, N, N_validate, N_test){
	
	N_total = N + N_validate + N_test
	Ctotal = (C*(C-1)/2)

# ----- create U 
	Uinv = matrix(0, nrow=r, ncol=r)
	for(i in 1:r){
		for(j in 1:r){
			Uinv[i,j] = rho.U^abs(i-j)
		}
	}
	diag(Uinv) = 1
	eoU = eigen(Uinv)
	Uinv.sqrt = eoU$vec%*%diag(eoU$val^(1/2))%*%t(eoU$vec)

# ----- create V
	Vinv = matrix(0, nrow=p, ncol=p)
	for(i in 1:p){
		for(j in 1:p){
			Vinv[i,j] = rho.V
		}
	}
	diag(Vinv) = 1 
	eoV = eigen(Vinv)
	Vinv.sqrt = eoV$vec%*%diag(eoV$val^(1/2))%*%t(eoV$vec)

# ---- create category means
	Mu.Array = array(0, dim=c(r,p,C))
	c1 = sample(0:(r-4), 1)
	c2 = sample(0:(p-4), 1)

	diff.mat1 = matrix(0,nrow=r, ncol=p)
	diff.mat1[(1:2)+c1, (1:2)+c2] = 1
	Mu.Array[,,1] = diff.mat1

	diff.mat2 = matrix(0,nrow=r, ncol=p)
	diff.mat2[(3:4)+c1, (3:4)+c2] = .5
	Mu.Array[,,2] = diff.mat2

	diff.mat3 = matrix(0, nrow=r, ncol=p)
	diff.mat3[(1:2)+c1, (1:2)+c2] = -2
	diff.mat3[(3:4)+c1, (3:4)+c2] = -.5
	Mu.Array[,,3] = diff.mat3

# ---- generate Matrix-normal data 
	dat.array = array(0, dim=c(r,p,N_total))
	class_total = numeric(length=N_total)
	for(jj in 1:N_total){
		class_total[jj] = sample(1:C, 1, prob=rep(1/C, C))
		dat.array[,,jj] = Uinv.sqrt%*%matrix(rnorm(r*p), nrow=r)%*%Vinv.sqrt + Mu.Array[,,class_total[jj]]
	}

# ---- divide into training, validation, and testing
	X = dat.array[,,1:N]
	X_val = dat.array[,,(N+1):(N+N_validate)]
	X_test = dat.array[,,(N+N_validate+1):N_total]
	class = class_total[1:N]
	class_val = class_total[(N+1):(N+N_validate)]
	class_test = class_total[(N+N_validate+1):N_total]


	return(list("X" = X, "class" = class, "X_val" = X_val, "X_test"= X_test, 
		"U" = solve(Uinv), "V" = solve(Vinv), "Mu_Array" = Mu.Array, "class_val" = class_val, "class_test"=class_test))
	
	}




# ------ used when r < p
CREATE_DAT_COV1_TRANS = function(r, p, C, rho.U, rho.V, N, N_validate, N_test){
	
	N_total = N + N_validate + N_test
	Ctotal = (C*(C-1)/2)

# ----- create U 
	Uinv = matrix(0, nrow=r, ncol=r)
	for(i in 1:r){
		for(j in 1:r){
			Uinv[i,j] = rho.U
		}
	} 
	diag(Uinv) = 1
	eoU = eigen(Uinv)
	Uinv.sqrt = eoU$vec%*%diag(eoU$val^(1/2))%*%t(eoU$vec)

# ----- create V
	Vinv = matrix(0, nrow=p, ncol=p)
	for(i in 1:p){
		for(j in 1:p){
			Vinv[i,j] = rho.V^abs(i-j)
		}
	}
	diag(Vinv) = 1 
	eoV = eigen(Vinv)
	Vinv.sqrt = eoV$vec%*%diag(eoV$val^(1/2))%*%t(eoV$vec)

# ---- create category means
	Mu.Array = array(0, dim=c(r,p,C))

	c1 = sample(0:(r-4), 1)
	c2 = sample(0:(p-4), 1)
	
	diff.mat1 = matrix(0,nrow=r, ncol=p)
	diff.mat1[(1:2)+c1, (1:2)+c2] = 1
	Mu.Array[,,1] = diff.mat1

	diff.mat2 = matrix(0,nrow=r, ncol=p)
	diff.mat2[(3:4)+c1, (3:4)+c2] = .5
	Mu.Array[,,2] = diff.mat2

	diff.mat3 = matrix(0, nrow=r, ncol=p)
	diff.mat3[(1:2)+c1, (1:2)+c2] = -2
	diff.mat3[(3:4)+c1, (3:4)+c2] = -.5
	Mu.Array[,,3] = diff.mat3

# ---- generate Matrix-normal data 
	dat.array = array(0, dim=c(r,p,N_total))
	class_total = numeric(length=N_total)
	for(jj in 1:N_total){
		class_total[jj] = sample(1:C, 1, prob=rep(1/C, C))
		dat.array[,,jj] = Uinv.sqrt%*%matrix(rnorm(r*p), nrow=r)%*%Vinv.sqrt + Mu.Array[,,class_total[jj]]
	}

# ---- divide into training, validation, and testing

	X = dat.array[,,1:N]
	X_val = dat.array[,,(N+1):(N+N_validate)]
	X_test = dat.array[,,(N+N_validate+1):N_total]
	class = class_total[1:N]
	class_val = class_total[(N+1):(N+N_validate)]
	class_test = class_total[(N+N_validate+1):N_total]


	return(list("X" = X, "class" = class, "X_val" = X_val, "X_test"= X_test, 
		"U" = solve(Uinv), "V" = solve(Vinv), "Mu_Array" = Mu.Array, "class_val" = class_val, "class_test"=class_test))
	
}


	