# -----------------------------------------
# Create data using Zhong and Suslick approaches 
# -----------------------------------------

CREATE_DAT_ZS = function(r, p=36, C=2, N, N_validate, N_test){
	
	N_total = N + N_validate + N_test

	Sigma = matrix(0, nrow=p, ncol=p)
	for(ooo in 1:p){
		for(lll in 1:p){
			Sigma[ooo,lll] = .5^abs(lll-ooo)
		}
	}

	eo = eigen(Sigma)
	Sigma.sqrt = eo$vec%*%diag(eo$val^.5)%*%t(eo$vec)

	dat.array = array(0, dim=c(r,p,N_total))
	class_total = numeric(length=N_total)
	for(jj in 1:N_total){
		class_total[jj] = sample(1:C, 1, prob=rep(1/C, C))
		dat.array[1,,jj] = Sigma.sqrt%*%matrix(rnorm(p), nrow=p) 
		dat.array[2,,jj] = sqrt(.5)*matrix(rnorm(p), ncol=p) + (abs(dat.array[1,,jj]) - .3*(class_total[jj]+1))
		dat.array[3:r,,jj] = matrix(rnorm(p*(r-2)), ncol=p)
	}


	X = dat.array[,,1:N]
	X_val = dat.array[,,(N+1):(N+N_validate)]
	X_test = dat.array[,,(N+N_validate+1):N_total]
	class = class_total[1:N]
	class_val = class_total[(N+1):(N+N_validate)]
	class_test = class_total[(N+N_validate+1):N_total]


	return(list("X" = X, "class" = class, "X_val" = X_val, "X_test"= X_test, 
		"class_val" = class_val, "class_test"=class_test))
	
}



CREATE_DAT_ZS_MOD = function(r, p=36, C=2, N, N_validate, N_test){
	
	N_total = N + N_validate + N_test

	Sigma = matrix(0, nrow=p, ncol=p)
	for(ooo in 1:p){
		for(lll in 1:p){
			Sigma[ooo,lll] = .5^abs(lll-ooo)
		}
	}

	eo = eigen(Sigma)
	Sigma.sqrt = eo$vec%*%diag(eo$val^.5)%*%t(eo$vec)

	dat.array = array(0, dim=c(r,p,N_total))
	class_total = numeric(length=N_total)
	for(jj in 1:N_total){
		class_total[jj] = sample(1:C, 1, prob=rep(1/C, C))
		dat.array[1,,jj] = Sigma.sqrt%*%matrix(rnorm(p), nrow=p) 
		if(class_total[jj]==1){
			dat.array[2,,jj] = sqrt(.5)*matrix(rnorm(p), ncol=p) + (abs(dat.array[1,,jj]) - c(rep(.5, 18), rep(0, 18)))
			} else {
			dat.array[2,,jj] = sqrt(.5)*matrix(rnorm(p), ncol=p) + (abs(dat.array[1,,jj]) - c(rep(0, 18), rep(.5, 18)))
			}
		dat.array[3:r,,jj] = matrix(rnorm(p*(r-2)), ncol=p)
	}


	X = dat.array[,,1:N]
	X_val = dat.array[,,(N+1):(N+N_validate)]
	X_test = dat.array[,,(N+N_validate+1):N_total]
	class = class_total[1:N]
	class_val = class_total[(N+1):(N+N_validate)]
	class_test = class_total[(N+N_validate+1):N_total]


	return(list("X" = X, "class" = class, "X_val" = X_val, "X_test"= X_test, 
		"class_val" = class_val, "class_test"=class_test))
	
}



