# --------------------------------------
# Compute Guo (2010) estimator 
# -------------------------------------

GUO_L1_NB = function(X, class, lambda, quiet=FALSE, warm=FALSE, beta=NULL){

# ---- preliminaries
	nc = count(class)[,2]
	N = sum(nc)
	C = length(nc)
	p = dim(X)[2]
	r = dim(X)[1]	

# ------ vecY from Guo (2010)
	vecY = t(apply(X, 3, as.vector))
	mean.array = ARRAY_CLASS_MEAN(X, class)
	X.c = CENTER_X(X, class, mean.array)
	vecX.c = t(apply(X.c, 3, as.vector))
	Sigma_diag = rep(0, dim(vecX.c)[2])
	for(ff in 1:dim(vecX.c)[2]){
		Sigma_diag[ff] = N^(-1)*crossprod(vecX.c[,ff])
	}
	mean.mat = t(apply(mean.array, 3, as.vector))
	class.mat = matrix(0, nrow=N, ncol=C)
	for (i in 1:N) {
		for (j in 1:C) {
			if (class[i]==j) {
				class.mat[i,j] = 1
			}
		}
	}    

# ------- precompute matrices 
	cp_class.mat = crossprod(class.mat)
	cp_class.mat_vecY = crossprod(class.mat, vecY)

	k.iter = 0
	resid = 1
	converged = FALSE
	
	if(warm){
		beta = beta
	} else {
 		beta = mean.mat
	}

# --------------------------------------------
#  iterating 
# --------------------------------------------

	while(!converged){

		temp.beta = matrix(0, nrow=dim(mean.mat)[1], ncol=dim(mean.mat)[2])
		
		for (j in 1:(r*p)){
			
			# ---- get off diagonals  
			G = matrix(0, nrow=C, ncol=C)
			for(l in 1:C){
				for(m in 1:C){
					# ---- threshold if below 1e-7; using (1e-8, 1e-10) resulted in singularities
					if(abs(beta[l,j] - beta[m,j]) < 1e-7){
						G[l,m] = -(abs(mean.mat[l,j] - mean.mat[m,j])^(-1))/1e-7
					} else {
						G[l,m] = -(abs(mean.mat[l,j] - mean.mat[m,j])^(-1))/abs(beta[l,j] - beta[m,j])
					}
				}
			}

			# --- get diagonals
			for(l in 1:C){
				out = 0 
				for(m in 1:C){
					if(m!=l){
						if (abs(beta[l,j] - beta[m,j]) < 1e-7) {
							out = out + (abs(mean.mat[l,j] - mean.mat[m,j])^(-1))/1e-7
						} else { 
							out = out + (abs(mean.mat[l,j] - mean.mat[m,j])^(-1))/abs(beta[l,j] - beta[m,j])
						}
					}
				}
				G[l,l] = out
			}

			temp.beta[,j] = solve(cp_class.mat + lambda*Sigma_diag[j]*G, cp_class.mat_vecY[,j])

		}

		resid = 0 
		numer = 0
		denom = 0

		for(c in 1:C){
			for(j in 1:(r*p)){
				numer = numer + abs(temp.beta[c, j] - beta[c,j])
				denom = denom + abs(beta[c,j])
			}
		}

		resid = numer/denom
		if(!quiet){
			cat("resid=", resid, "k.iter=", k.iter, "\n")
		}
		if(resid < 1e-6){
			converged=TRUE
		} else { 
			if(k.iter > 1000){
				converged=TRUE
			}
		} 

		beta = temp.beta
		k.iter = k.iter + 1
	
	}
	
	# ---- threshold final iterates for mean equality
	beta.final <- beta 
	for(c in 1:(C-1)){
		for(kk in (c+1):C){
			for(j in 1:(r*p)){
				if(abs(beta.final[c,j] - beta.final[kk,j]) <= 1e-7){
	 				beta[kk, j] <- beta[c,j]
				}
			}
		}
	}

	return(list("mean.matrix" = beta, "Sigma_diag" = diag(Sigma_diag)))
	
}
