

# ------------------------------------------------------------------------
# Class fusion with covariance known and non-separable  (Xu et al. 2015)
# -----------------------------------------------------------------------

XU_SIGMA_GRID = function(X, class, Sigma, Omega, lambda, X_val, class_val, X_test, class_test, Mu_Array = Mu_Array, rho.input = NULL){

# --- preliminaries --- 
	nc = count(class)[,2]
	N = sum(nc)
	C = length(nc)
	q = dim(X)[2]*dim(X)[1]	
	pi1 <- nc[1]/N
	pi2 <- nc[2]/N
	pi3 <- nc[3]/N
	vecY = t(apply(X, 3, as.vector))
	mean.array = ARRAY_CLASS_MEAN(X, class)
	mean.mat = apply(mean.array, 3, as.vector)
	val.set = rep(0, length(lambda))
	test.set = rep(0, length(lambda))
	TPR.set = rep(0, length(lambda))
	TNR.set = rep(0, length(lambda))
	beta.full = array(0, dim=c(q, C, length(lambda)))
	vecX_test = apply(X_test, 3, as.vector)
	vecX_val = apply(X_val, 3, as.vector)

# ---- weight matrices ----
	w12 <- 1/abs(mean.mat[,1] - mean.mat[,2])
	w13 <- 1/abs(mean.mat[,1] - mean.mat[,3])
	w23 <- 1/abs(mean.mat[,2] - mean.mat[,3])
	mean.tot <- apply(apply(X, 3, as.vector),1, mean)

# ---- rho -----
	if(is.null(rho.input)){
			rho = (4/3)*(min(nc)/N)*(1/max(eigen(Sigma)$val))*.5
		} else {
			rho = (4/3)*(min(nc)/N)*(1/rho.input)*.5
		}
# ---- Gammas  ----- 
	Gamma12 <- matrix(0, nrow=q, ncol=1)
	Gamma13 <- matrix(0, nrow=q, ncol=1)
	Gamma23 <- matrix(0, nrow=q, ncol=1)

	mu1 <- matrix(mean.tot, ncol=1)
	mu2 <- matrix(mean.tot, ncol=1)
	mu3 <- matrix(mean.tot, ncol=1)
	tot.diff <- sum((mean.mat[,1] - mean.mat[,2])^2) + sum((mean.mat[,1] - mean.mat[,3])^2) + sum((mean.mat[,2] - mean.mat[,3])^2)
	tot.diff <- sqrt(tot.diff/3)

# ----------- clip function 
	clip <- function(X, W){
		return(pmin(abs(X), W)*sign(X))
	}

# ------- AMA based on equations in paper 
	for(j in 1:length(lambda)){

		k.iter <- 1
		iterating = TRUE
		Gamma12 = clip(Gamma12 - rho*mu1 + rho*mu2, lambda[j]*w12)
		Gamma13 = clip(Gamma13 - rho*mu1 + rho*mu3, lambda[j]*w13)
		Gamma23 = clip(Gamma23 - rho*mu2 + rho*mu3, lambda[j]*w23)


		while(iterating){

			mu1new =  mean.mat[,1] + .5/pi1*crossprod(Sigma, Gamma12 + Gamma13)
			mu2new =  mean.mat[,2] + .5/pi2*crossprod(Sigma, Gamma23 - Gamma12)
			mu3new =  mean.mat[,3] + .5/pi3*crossprod(Sigma, -Gamma12 -Gamma23)

			Gamma12new = clip(Gamma12 - rho*mu1new + rho*mu2new, lambda[j]*w12)
			Gamma13new = clip(Gamma13 - rho*mu1new + rho*mu3new, lambda[j]*w13)
			Gamma23new = clip(Gamma23 - rho*mu2new + rho*mu3new, lambda[j]*w23)

			rk <- sum((mu1 - mu1new)^2) + sum((mu2 - mu2new)^2) + sum((mu3 - mu3new)^2)
			dk <- sum((Gamma12 - Gamma12new)^2) + sum((Gamma13 - Gamma13new)^2) + sum((Gamma23 - Gamma23new)^2)
			# if(k.iter %% 100 == 0){
			# 	cat(k.iter, "; rk = ", rk, "(threshold = ", (1e-5*tot.diff), "); dk = ", dk, "\n")
			# }
			if(rk < (1e-5*tot.diff) & dk < 1e-5){
				iterating = FALSE
			} 

			mu1 <- mu1new
			mu2 <- mu2new
			mu3 <- mu3new
			Gamma12 <- Gamma12new
			Gamma13 <- Gamma13new
			Gamma23 <- Gamma23new

			if(k.iter > 2500){
				iterating = FALSE
			}

			k.iter <- k.iter + 1
		}

	# ------ get thresholded values 
		mu1new =  mean.mat[,1] + .5/pi1*crossprod(Sigma,  Gamma12 + Gamma13)
		mu2new =  mean.mat[,2] + .5/pi2*crossprod(Sigma,  Gamma23 - Gamma12)
		mu3new =  mean.mat[,3] + .5/pi3*crossprod(Sigma, - Gamma12 - Gamma23)

		theta12 = pmax(abs(mu1 - mu2 - rho^(-1)*Gamma12) - (lambda[j]/rho)*w12,0)*sign(mu1 - mu2 - rho^(-1)*Gamma12)
		theta13 = pmax(abs(mu1 - mu3 - rho^(-1)*Gamma13) - (lambda[j]/rho)*w13,0)*sign(mu1 - mu3 - rho^(-1)*Gamma13)
		theta23	= pmax(abs(mu2 - mu3 - rho^(-1)*Gamma23) - (lambda[j]/rho)*w23,0)*sign(mu2 - mu3 - rho^(-1)*Gamma23)

		beta <- t(cbind(mu1new, mu2new, mu3new))
	# ---- use our thresholding procedure 
		for (kk in 1:q) {
			if(theta12[kk]  == 0){
		 		beta[2,kk] <- beta[1,kk]
			}
			if(theta13[kk] == 0){
				beta[3,kk] <- beta[1,kk]
			}
			if(theta23[kk] == 0){
				beta[3,kk] <- beta[2,kk]
			}
		}

	# ---- evaluate validation and testing set accuracy
		val.set[j] <-  NORM_CLASSIFICATION_LDA(X=vecX_val, class=class_val, pihat = c(pi1, pi2, pi3), Mean = t(beta), SigmaInv = Omega)$misclass
		test.set[j] <-  NORM_CLASSIFICATION_LDA(X=vecX_test, class=class_test, pihat = c(pi1, pi2, pi3), Mean = t(beta), SigmaInv = Omega)$misclass
		mean.array.temp = array(0, dim=c(dim(X)[1], dim(X)[2], C))
		for(c in 1:C){
			mean.array.temp[,,c] = matrix(beta[c,], nrow=r, byrow=F)
		}

		temp = VS_CHECK(Mu_Array = Mu_Array, mean.array = mean.array.temp)
		TPR.set[j] <- temp$TPR
		TNR.set[j] <- temp$TNR
		#cat("Val =" = val.set[j], "; Test = ", test.set[j], "TPR = ", TPR.set[j], "TNR = ", TNR.set[j], "\n")
		#cat("through ", j, "\n")

	}

	return(list("Val.mat" = val.set, "Test.mat" = test.set, "TPR.set" = TPR.set, "TNR.set" = TNR.set))
	
}


# ------------------------------------------------------------------------
# Class fusion with covariance known and separable  (Xu et al. 2015)
# -----------------------------------------------------------------------

KRON_XU_GRID = function(X, class, U, V, lambda1, quiet=TRUE,
	Xval=NULL, classval= NULL,
	k.iter = 100, m.tol=1e-5){

# ------ Preliminaries; order n1 > .. > nC
	nc.orig = count(class)[,2]
	N = sum(nc.orig)

	class.orig = class
	class.val.orig =  classval

	out = rank(-nc.orig, ties.method = "first")
	class = rep(0, N)
	for (gg in 1:N) {
		class[gg] = out[class.orig[gg]]
	}

	nc = count(class)[,2]

	C = length(nc)
	K = (C*(C-1))/2
	N = sum(nc)
	p = dim(X)[2]
	r = dim(X)[1]
	pihat = nc/N


	classval = rep(0, dim(Xval)[3])
	for (gg in 1:dim(Xval)[3]) {
		classval[gg] = out[class.val.orig[gg]]
	}

	Vinv = solve(V)
	Uinv = solve(U)
	eoU = eigen(U)
	eoV = eigen(V)
	rho.max = (4*min(eoU$val)*min(eoV$val)*min(nc))/(C*N)
	rho = rho.max/10

# ---- sample mean array
	mean.array = array(0, dim=c(r,p,C))
	for (k in 1:C) {
		mean.array[,,k] = apply(X[,,class==k], c(1,2), mean)
	}

# ---- weight matrix
	weightmat = matrix(0, nrow=K*r, ncol=p)
	for (c in 1:(C-1)) {
		for(cc in (c+1):C){
		k = (C - 1)*(c-1) + (cc-1) - (c*(c-1)/2)
		weightmat[((k-1)*r + 1):(k*r), ] = 1/abs(mean.array[,,c] - mean.array[,,cc])
		}
	}

	G.mat = matrix(0, nrow=length(lambda1),  ncol=1)


	if(!is.null(Xval)){
		Val = matrix(0, nrow=length(lambda1), ncol=1)
	}

	outer.converge=FALSE
	meanstacked = STACKED(mean.array)
	M.update = mean.array

# ---- iterate 
	for(j in 1:length(lambda1)){

		if(lambda1[j]!=0){

			# ----- update means
				D = matrix(0, nrow=K*r, ncol=p)
				out = M_Update_Rcpp(Mean=meanstacked,
					 D=D, Uinv=Uinv, Vinv=Vinv, nc = nc, rho=rho, weightmat = weightmat,
				  lambda=lambda1[j], C=C, r=r, mtol = m.tol)

			# ---- store D for warm starts
				D = out$D

			# ---- threshold mean
				M = THRESH_MEAN(out$M, out$G, C)
				M.update = array(0, dim=c(r, p, C))
				for(c in 1:C){
					M.update[,,c] = M[((c-1)*r +1):(c*r),]
				}

		} else {
				M.update = mean.array
		}

	# ----  if val is non-null, compute misclassification rate
		if(!is.null(Xval)){
			Val[j,1] = MN_CLASSIFICATION(X=Xval, class=classval, M.update, U, V, pi.list=pihat, C=C)$misclass
		} else {
			Val = NULL
		}

		G.mat[j,1] = 0
		for(jj in 1:(C-1)){
			for(kk in (jj+1):C){
				G.mat[j,1] = G.mat[j,1] + sum(abs(M.update[,,jj] - M.update[,,kk])==0)
			}
		}

	}

	return(list("Val.mat" = Val, "G.mat" = G.mat))

}




KRON_XU = function(X, class, U, V, lambda1, quiet=TRUE,
	Xval=NULL, classval= NULL,
	k.iter = 100, m.tol=1e-5){

	nc.orig = count(class)[,2]
	N = sum(nc.orig)

	class.orig = class
	class.val.orig =  classval

	out = rank(-nc.orig, ties.method = "first")
	class = rep(0, N)
	for (gg in 1:N) {
		class[gg] = out[class.orig[gg]]
	}

	nc = count(class)[,2]

	C = length(nc)
	K = (C*(C-1))/2
	N = sum(nc)
	p = dim(X)[2]
	r = dim(X)[1]
	pihat = nc/N

	classval = rep(0, dim(Xval)[3])
	for (gg in 1:dim(Xval)[3]) {
		classval[gg] = out[class.val.orig[gg]]
	}

	Vinv = solve(V)
	Uinv = solve(U)
	eoU = eigen(U)
	eoV = eigen(V)
	rho.max = (4*min(eoU$val)*min(eoV$val)*min(nc))/(C*N)
	rho = rho.max/10

	mean.array = array(0, dim=c(r,p,C))
	for (k in 1:C) {
		mean.array[,,k] = apply(X[,,class==k], c(1,2), mean)
	}

	weightmat = matrix(0, nrow=K*r, ncol=p)
	for (c in 1:(C-1)) {
		for(cc in (c+1):C){
		k = (C - 1)*(c-1) + (cc-1) - (c*(c-1)/2)
		weightmat[((k-1)*r + 1):(k*r), ] = 1/abs(mean.array[,,c] - mean.array[,,cc])
		}
	}

	G.mat = matrix(0, nrow=length(lambda1),  ncol=1)


	if(!is.null(Xval)){
		Val = matrix(0, nrow=length(lambda1), ncol=1)
	}

	outer.converge=FALSE
	meanstacked = STACKED(mean.array)
	M.update = mean.array

	for(j in 1:length(lambda1)){

		if(lambda1[j]!=0){

				# ---- update means
				D = matrix(0, nrow=K*r, ncol=p)
				out = M_Update_Rcpp(Mean=meanstacked,
					 D=D, Uinv=Uinv, Vinv=Vinv, nc = nc, rho=rho, weightmat = weightmat,
				  lambda=lambda1[j], C=C, r=r, mtol = m.tol)

				# --- store D for warm starts
				D = out$D

				# --- threshold mean
				M = THRESH_MEAN(out$M, out$G, C)

				M.update = array(0, dim=c(r, p, C))
				for(c in 1:C){
					M.update[,,c] = M[((c-1)*r +1):(c*r),]
				}

		} else {
				M.update = mean.array
		}

		## If Valset, compute misclassification rate
		if(!is.null(Xval)){
			Val[j,1] = MN_CLASSIFICATION(X=Xval, class=classval, M.update, U, V, pi.list=pihat, C=C)$misclass
		} else {
			Val = NULL
		}

		G.mat[j,1] = 0
		for(jj in 1:(C-1)){
			for(kk in (jj+1):C){
				G.mat[j,1] = G.mat[j,1] + sum(abs(M.update[,,jj] - M.update[,,kk])==0)
			}
		}

	}

	return(list("Val.mat" = Val, "Mean" = M.update))

}


