# --------------------------------------
# Function to compute PMN(m) from manuscript 
# ------------------------------------

ORACLE_MEAN_GRID = function(X, class, Mean, lambda2, quiet=TRUE, 
	Xval=NULL, classval= NULL,
	k.iter = 100, cov.tol=1e-5, full.tol=1e-6){
	
# ------  preliminaries
	nc = count(class)[,2]
	C = length(nc)
	K = (C*(C-1))/2
	N = sum(nc)
	p = dim(X)[2]
	r = dim(X)[1]
	pihat = nc/N
	
# ------ initialize U and V at diagonal
	out = MN_MLE(X, class)
	V.unscaled = diag(diag(out$V))
	U.unscaled = diag(diag(out$U))
	out1 = sum(abs(U.unscaled))
	UInit = (r/out1)*U.unscaled
	VInit = (out1/r)*V.unscaled
	
	Vinv = solve(VInit)
	Uinv = solve(UInit)

	Val = matrix(0, nrow=1,  ncol=length(lambda2))
	U.mat = matrix(0, nrow=1,  ncol=length(lambda2))
	V.mat = matrix(0, nrow=1,  ncol=length(lambda2))

	meanstacked = STACKED(Mean)
	mean.array = Mean
	weightmat = matrix(0, nrow=K*r, ncol=p)
	D = matrix(0, nrow=K*r, ncol=p)			
	
	init.objfncval = EVAL_OBJ_FUNC(X, class, Mean, UInit, VInit,  weightmat, lambda1=0, lambda2)$out


# ------- center and compute path of covariances 
	Xc = CENTER_X(X, class, Mean)
    S.v = V_SAMPLE(X.c=Xc, U= UInit)
  	V.temp = glassopath(S.v, rholist=lambda2, trace=0, thr=cov.tol, penalize.diagonal=TRUE)$wi

  	for(i in 1:length(lambda2)){
  	
		lambda.u = (lambda2[i]*sum(abs(V.temp[,,i]))/p)
  		S.u = U_SAMPLE(X.c = Xc, V = V.temp[,,i])
  		U.temp = glasso(S.u, rho = lambda.u, thr=cov.tol, trace=0, penalize.diagonal=TRUE)$wi

		out.inner = GENERAL_ITER(X, class, lambda1 = 0, lambda2 = lambda2[i],
				quiet=quiet, UInit = UInit, VInit = VInit, U=U.temp, V=V.temp[,,i],
						  meanstacked = meanstacked, mean.array= mean.array, M.update = Mean, D=D, weightmat=weightmat,
						  full.tol = full.tol, cov.tol=cov.tol, m.tol=1e-5, init.objfncval=init.objfncval, k.iter=k.iter)
				
  	# ----- if val set is non-NULL, compute misclassification rate
  		if (!is.null(Xval)) {
			Val[1,i] = MN_CLASSIFICATION(X=Xval, class=classval, Mean, out.inner$U, out.inner$V, pi.list=pihat, C=C)$misclass
		} else {
			Val = NULL
		}

		U.mat[1,i] = sum(out.inner$U==0)
		V.mat[1,i] = sum(out.inner$V==0)

	}
	
	return(list("Val.mat" = Val, "U.mat" = U.mat, "V.mat" = V.mat))

}



ORACLE_MEAN_SINGLE = function(X, class, Mean, lambda2, quiet=TRUE, 
	Xval=NULL, classval= NULL,
	k.iter = 100, cov.tol=1e-5, full.tol=1e-6){
	
# -----  preliminaries
	nc = count(class)[,2]
	C = length(nc)
	K = (C*(C-1))/2
	N = sum(nc)
	p = dim(X)[2]
	r = dim(X)[1]
	pihat = nc/N
	
# ----- initialize U and V at diagonal	
	out = MN_MLE(X, class)
	V.unscaled = diag(diag(out$V))
	U.unscaled = diag(diag(out$U))
	out1 = sum(abs(U.unscaled))
	UInit = (r/out1)*U.unscaled
	VInit = (out1/r)*V.unscaled
	
	Vinv = solve(VInit)
	Uinv = solve(UInit)

	Val = matrix(0, nrow=1,  ncol=length(lambda2))
	U.mat = matrix(0, nrow=1,  ncol=length(lambda2))
	V.mat = matrix(0, nrow=1,  ncol=length(lambda2))

	meanstacked = STACKED(Mean)
	mean.array = Mean
	weightmat = matrix(0, nrow=K*r, ncol=p)
	D = matrix(0, nrow=K*r, ncol=p)			
	init.objfncval = EVAL_OBJ_FUNC(X, class, Mean, UInit, VInit,  weightmat, lambda1=0, lambda2)$out


# ----- center and compute path of covariances 
	Xc = CENTER_X(X, class, Mean)
    S.v = V_SAMPLE(X.c = Xc, U = UInit)
  	V.temp = glassopath(S.v, rholist=lambda2, trace=0, thr=cov.tol, penalize.diagonal=TRUE)$wi

  	for(i in 1:length(lambda2)){
  	
		lambda.u = (lambda2[i]*sum(abs(V.temp[,,i]))/p)
  		S.u = U_SAMPLE(X.c = Xc, V = V.temp[,,i])
  		U.temp = glasso(S.u, rho = lambda.u, thr=cov.tol, trace=0, penalize.diagonal=TRUE)$wi

		out.inner = GENERAL_ITER(X, class, lambda1 = 0, lambda2 = lambda2[i],
				quiet=quiet, UInit = UInit, VInit = VInit, U = U.temp, V=V.temp[,,i],
						  meanstacked = meanstacked, mean.array = mean.array, M.update = Mean, D = D, weightmat=weightmat,
						  full.tol = full.tol, cov.tol=cov.tol, m.tol=1e-5, init.objfncval=init.objfncval, k.iter=k.iter)
				
  	# ---- if val set is non-NULL, compute misclassification rate
  		if (!is.null(Xval)) {
			Val[1,i] = MN_CLASSIFICATION(X=Xval, class=classval, Mean, out.inner$U, out.inner$V, pi.list=pihat, C=C)$misclass
		} else {
			Val = NULL
		}

	}


	
return(list("Val" = Val, "U" = out.inner$U, "V" = out.inner$V))

}


