# --------------------------------------------
# Vector-valued SURE screening (3-classes only)
# --------------------------------------------

SURE_SCREEN = function(X, class, X_val, class_val, X_test, class_test){
	
# ---- preliminaries
	N = dim(X)[3]
	r = dim(X)[1]
	p = dim(X)[2]
	C = length(unique(class))

# ----- standardize per Pan et al. 
	mean.array = ARRAY_CLASS_MEAN(X=X, class=class)
	X.c = CENTER_X(X, class, mean.array)
	sd.train = apply(X.c, c(1,2), sd)

	# --- replace X's with standardized version 
	for(j in 1:dim(X)[3]){
		X[,,j] <- X[,,j]/sd.train
	}
	X.stand <- X

	for(j in 1:dim(X_val)[3]){
		X_val[,,j] <- X_val[,,j]/sd.train
	}
	X_val.stand <- X_val

	# we dont rename X_test to avoid copying large matrix 
	for(j in 1:dim(X_test)[3]){
		X_test[,,j] <- X_test[,,j]/sd.train
	}

	# --- recompute mean and adjust validation and testing sets 
	mean.array = ARRAY_CLASS_MEAN(X = X.stand, class=class)
	mean.vec = apply(mean.array, 3, as.vector)
	X.c = CENTER_X(X.stand, class, mean.array)
	vec.sd.train = as.vector(sd.train)

	# -- use vecX.c for covariance estimation 
	vecX.c = t(apply(X.c, 3, as.vector))

	# -- use others for classification/validation
	vecX_val = apply(X_val, 3, as.vector)
	vecX_test = apply(X_test, 3, as.vector)

# ----- 1-2

	val12 = rep(1, 25)
	pihat = rep(0, 2)
	N12 = sum(count(class)[1:2,2])
	pihat[1] = length(which(class==1))/N12
	pihat[2] = length(which(class==2))/N12

	for(k in 2:25){

		out12 = abs(mean.array[,,1] - mean.array[,,2])
		out12.thresh = matrix(0, nrow=r, ncol=p)
		
		for(j in 1:k){
			check = which(out12 == max(out12[out12!=0]), arr.ind=T)
			out12.thresh[check[1], check[2]] <- 1 	
			out12[check[1], check[2]] <- 0
		}

		active.set = which(as.vector(out12.thresh)==1)
		mean.vec.as = mean.vec[active.set, 1:2]
		vecX.c.as = vecX.c[,active.set]
		SigmaInv = solve(N^(-1)*crossprod(vecX.c.as))
		val12[k]= NORM_CLASSIFICATION_LDA(X=vecX_val[active.set, which(class_val==1 | class_val==2)], class=class_val[which(class_val==1 | class_val==2)], pihat = pihat, Mean = mean.vec.as, SigmaInv = SigmaInv)$misclass
	}

	# -- find smallest model size which minimizes validation error
	ind.min12 = min(which(val12 == min(val12)))

# ----- 1-3

	val13 = rep(1, 25)
	pihat = rep(0, 2)
	N13 = sum(count(class)[c(1,3),2])
	pihat[1] = length(which(class==1))/N13
	pihat[2] = length(which(class==3))/N13
	inner.class = ifelse(class_val[which(class_val==1 | class_val==3)]==1, 1, 2)
	val13[1] = 1; 

	for(k in 2:25){

		out13 = abs(mean.array[,,1] - mean.array[,,3])
		out13.thresh = matrix(0, nrow=r, ncol=p)
		
		for(j in 1:k){
			check = which(out13 == max(out13[out13!= 0]), arr.ind=T)
			out13.thresh[check[1], check[2]] <- 1 	
			out13[check[1], check[2]] <- 0 	
		}

		active.set = which(as.vector(out13.thresh)==1)
		mean.vec.as = mean.vec[active.set, c(1,3)]
		vecX.c.as = vecX.c[,active.set]
		SigmaInv = solve(N^(-1)*crossprod(vecX.c.as))
		val13[k]= NORM_CLASSIFICATION_LDA(X=vecX_val[active.set, which(class_val==1 | class_val==3)], class=inner.class, pihat = pihat, Mean = mean.vec.as, SigmaInv = SigmaInv)$misclass
		
	}

	ind.min13 = min(which(val13 == min(val13)))

# ----- 2-3
	val23 = rep(1, 25)
	pihat = rep(0, 2)
	N23 = sum(count(class)[c(2,3),2])
	pihat[1] = length(which(class==2))/N23
	pihat[2] = length(which(class==3))/N23
	inner.class = ifelse(class_val[which(class_val==2 | class_val==3)]==2, 1, 2)
	val23[1] = 1; 

	for(k in 2:25){

		out23 = abs(mean.array[,,2] - mean.array[,,3])
		out23.thresh = matrix(0, nrow=r, ncol=p)
		
		for(j in 1:k){
			check = which(out23 == max(out23[out23!= 0]), arr.ind=T)
			out23.thresh[check[1], check[2]] <- 1 	
			out23[check[1], check[2]] <- 0 	
		}

		active.set = which(as.vector(out23.thresh)==1)
		mean.vec.as = mean.vec[active.set, c(2,3)]
		vecX.c.as = vecX.c[,active.set]
		SigmaInv = solve(N^(-1)*crossprod(vecX.c.as))
		val23[k]= NORM_CLASSIFICATION_LDA(X=vecX_val[active.set, which(class_val==2 | class_val==3)], class=inner.class, pihat = pihat, Mean = mean.vec.as, SigmaInv = SigmaInv)$misclass
	}
	ind.min23 = min(which(val23 == min(val23)))

# ------------------------------------------------
#  Fit full models + community vote 
# ------------------------------------------------

# ----- 1-2 

	out12 = abs(mean.array[,,1] - mean.array[,,2])
	out12.thresh = matrix(0, nrow=r, ncol=p)
	
	for(j in 1:ind.min12){
		check = which(out12 == max(out12[out12!= 0]), arr.ind=T)
		out12.thresh[check[1], check[2]] <- 1 	
		out12[check[1], check[2]] <- 0 	
	}

	active.set.12 = which(as.vector(out12.thresh)==1)
	mean.vec.diff12 = mean.vec[active.set.12, 1] - mean.vec[active.set.12, 2]
	vecX.c.as = vecX.c[,active.set.12]
	beta12 = solve(N^(-1)*crossprod(vecX.c.as), mean.vec.diff12)

# ----- 1-3

	out13 = abs(mean.array[,,1] - mean.array[,,3])
	out13.thresh = matrix(0, nrow=r, ncol=p)
	
	for(j in 1:ind.min13){
		check = which(out13 == max(out13[out13!= 0]), arr.ind=T)
		out13.thresh[check[1], check[2]] <- 1 	
		out13[check[1], check[2]] <- 0 	
	 }
	active.set.13 = which(as.vector(out13.thresh)==1)
	mean.vec.diff13 = mean.vec[active.set.13, 1] - mean.vec[active.set.13, 3]
	 
	vecX.c.as = vecX.c[,active.set.13]
	beta13 = solve(N^(-1)*crossprod(vecX.c.as), mean.vec.diff13)

# ----- 2-3

	out23 = abs(mean.array[,,2] - mean.array[,,3])
	out23.thresh = matrix(0, nrow=r, ncol=p)
	
	for(j in 1:ind.min23){
		check = which(out23 == max(out23[out23!= 0]), arr.ind=T)
		out23.thresh[check[1], check[2]] <- 1 	
		out23[check[1], check[2]] <- 0 	
	}
	active.set.23 = which(as.vector(out23.thresh)==1)
	mean.vec.diff23 = mean.vec[active.set.23, 2] - mean.vec[active.set.23, 3]
	vecX.c.as = vecX.c[,active.set.23]
	beta23 = solve(N^(-1)*crossprod(vecX.c.as), mean.vec.diff23)

# --- Community vote
	
	test.length = dim(vecX_test)[2]
	test.votes = matrix(0, nrow=test.length, ncol=3)
	class.rate = rep(0, test.length)

	for(j in 1:test.length){
		# ---  1-2
		test12 = crossprod((vecX_test[active.set.12, j] - (mean.vec[active.set.12, 1] + mean.vec[active.set.12, 2])/2), beta12)
		if(test12 > 0){
			test.votes[j, 1] <- test.votes[j, 1] + 1
		} else{
			test.votes[j, 2] <- test.votes[j, 2] + 1
		}

		# ----- 1-3
		test13 = crossprod((vecX_test[active.set.13, j] - (mean.vec[active.set.13, 1] + mean.vec[active.set.13, 3])/2), beta13)
		if(test13 > 0){
			test.votes[j, 1] <- test.votes[j, 1]+ 1
		} else{
		 test.votes[j, 3] <- test.votes[j, 3] + 1
		}

		# ---- 2-3
		test23 = crossprod((vecX_test[active.set.23, j] - (mean.vec[active.set.23, 2] + mean.vec[active.set.23, 3])/2), beta23)
		if(test23 > 0){
			test.votes[j, 2] <- test.votes[j, 2] + 1
		} else{
			test.votes[j, 3] <- test.votes[j, 3] + 1
		}

		class.rate[j] = which.max(test.votes[j,]) != class_test[j]
	}

# --- Clean output 
	tuned.misclass.rate = mean(class.rate)

	## retun tuned_mean_array: 
	G12 = rep(0, r*p)
	G12[active.set.12] = 1

	G13 = rep(0, r*p)
	G13[active.set.13] = 1

	G23 = rep(0, r*p)
	G23[active.set.23] = 1

	G.array.tuned = array(0, dim=c(r, p, 2, 2))
	G.array.tuned[,,1, 1] = matrix(G12, byrow=FALSE, nrow=r, ncol=p)
	G.array.tuned[,,1, 2] = matrix(G13, byrow=FALSE, nrow=r, ncol=p)
	G.array.tuned[,,2, 2] = matrix(G23, byrow=FALSE, nrow=r, ncol=p)


# ------------------------------------------------
# Matrix-normal post screen 
# -----------------------------------------------

# ------ 1-2
	val12 = rep(1, 25)
	pihat = rep(0, 2)
	N12 = sum(count(class)[1:2,2])
	pihat[1] = length(which(class==1))/N12
	pihat[2] = length(which(class==2))/N12
	inner.class = ifelse(class_val[which(class_val==1 | class_val==2)]==1, 1, 2)
	val12[1] = 1; 

	for(k in 2:25){

		out12 = abs(mean.array[,,1] - mean.array[,,2])
		out12.thresh = matrix(0, nrow=r, ncol=p)
		
		for(j in 1:k){
			check = which(out12 == max(out12[out12!=0]), arr.ind=T)
			out12.thresh[check[1], check[2]] <- 1 	
			out12[check[1], check[2]] <- 0
		 }

		 active.set.rows = which(rowSums(out12.thresh)!=0)
		 active.set.columns = which(colSums(out12.thresh)!=0)

		 if(length(active.set.rows)>1 & length(active.set.columns)>1){
			
			out = MN_MLE(X = X.stand[active.set.rows, active.set.columns, class!=3], class=ifelse(class[which(class==1 | class==2)]==1, 1, 2))
			U.mle = out$U
			V.mle = out$V
			mean.array.mle = array(0, dim=c(r, p, 2))
			for(jj in 1:r){
				for(kk in 1:p){
					if(out12.thresh[jj,kk]==0){
						mean.array.mle[jj,kk,1] <- mean.array.mle[jj,kk,2] <- mean(X.stand[jj,kk,class==1 | class==2])
					} else {
						mean.array.mle[jj,kk,1] <- mean(X.stand[jj,kk,class==1])
						mean.array.mle[jj,kk,2] <- mean(X.stand[jj,kk,class==2])
					}
				}
			}

			mean.array.mle = mean.array.mle[active.set.rows, active.set.columns, ]
			val12[k]=MN_CLASSIFICATION(X=X_val.stand[active.set.rows,active.set.columns, which(class_val==1 | class_val==2)], class=class_val[which(class_val==1 | class_val==2)], pi.list = pihat, M = mean.array.mle, U = U.mle, V= V.mle)$misclass
		

		} else {
		
			active.set = which(out12.thresh==1)
			mean.vec.as = mean.vec[active.set, c(1,2)]
			vecX.c.as = vecX.c[,active.set]
			SigmaInv = solve(N^(-1)*crossprod(vecX.c.as))
			val12[k]= NORM_CLASSIFICATION_LDA(X=vecX_val[active.set, which(class_val==1 | class_val==2)], class=class_val[which(class_val==1 | class_val==2)], pihat = pihat, Mean = mean.vec.as, SigmaInv = SigmaInv)$misclass
		
		}
	
	}

	# ------ store for later use
	ind.min12 = min(which(val12 == min(val12)))

# ------ 1-3
	val13 = rep(1, 25)
	pihat = rep(0, 2)
	N13 = sum(count(class)[c(1,3),2])
	pihat[1] = length(which(class==1))/N13
	pihat[2] = length(which(class==3))/N13
	inner.class = ifelse(class_val[which(class_val==1 | class_val==3)]==1, 1, 2)
	val13[1] = 1; 

	for(k in 2:25){
		out13 = abs(mean.array[,,1] - mean.array[,,3])
		out13.thresh = matrix(0, nrow=r, ncol=p)
		
		for(j in 1:k){
			check = which(out13 == max(out13[out13!=0]), arr.ind=T)
			out13.thresh[check[1], check[2]] <- 1 	
			out13[check[1], check[2]] <- 0
		 }

		 active.set.rows = which(rowSums(out13.thresh)!=0)
		 active.set.columns = which(colSums(out13.thresh)!=0)
		 if(length(active.set.rows)>1 & length(active.set.columns)>1){

			out = MN_MLE(X = X.stand[active.set.rows, active.set.columns,class==1 | class==3], class=ifelse(class[which(class==1 | class==3)]==1, 1, 2))
			U.mle = out$U
			V.mle = out$V

			mean.array.mle = array(0, dim=c(r, p, 2))
			for(jj in 1:r){
				for(kk in 1:p){
					if(out13.thresh[jj,kk]==0){
						mean.array.mle[jj,kk,1] <- mean.array.mle[jj,kk,2] <- mean(X.stand[jj,kk,class==1 | class==3])
					} else {
						mean.array.mle[jj,kk,1] <- mean(X.stand[jj,kk,class==1])
						mean.array.mle[jj,kk,2] <- mean(X.stand[jj,kk,class==3])
					}
				}
			}
			mean.array.mle = mean.array.mle[active.set.rows, active.set.columns, ]
			val13[k]=MN_CLASSIFICATION(X=X_val.stand[active.set.rows,active.set.columns, which(class_val==1 | class_val==3)], class=inner.class, pi.list = pihat, M = mean.array.mle, U = U.mle, V= V.mle)$misclass

		} else {
			active.set = which(out13.thresh==1)
			mean.vec.as = mean.vec[active.set, c(1,3)]
			vecX.c.as = vecX.c[,active.set]
			SigmaInv = solve(N^(-1)*crossprod(vecX.c.as))
			val13[k]= NORM_CLASSIFICATION_LDA(X=vecX_val[active.set, which(class_val==1 | class_val==3)], class=inner.class, pihat = pihat, Mean = mean.vec.as, SigmaInv = SigmaInv)$misclass
		}
	
	}

		## store for later use
		ind.min13 = min(which(val13 == min(val13)))

# ------ 2-3
	val23 = rep(1, 25)
	pihat = rep(0, 2)
	N23 = sum(count(class)[c(2,3),2])
	pihat[1] = length(which(class==2))/N23
	pihat[2] = length(which(class==3))/N23
	inner.class = ifelse(class_val[which(class_val==2 | class_val==3)]==2, 1, 2)
	val23[1] = 1; 

	for(k in 2:25){

		out23 = abs(mean.array[,,2] - mean.array[,,3])
		out23.thresh = matrix(0, nrow=r, ncol=p)
		
		for(j in 1:k){
			check = which(out23 == max(out23[out23!=0]), arr.ind=T)
			out23.thresh[check[1], check[2]] <- 1 	
			out23[check[1], check[2]] <- 0
		 }

		 active.set.rows = which(rowSums(out23.thresh)!=0)
		 active.set.columns = which(colSums(out23.thresh)!=0)
		 if(length(active.set.rows)>1 & length(active.set.columns)>1){

			out = MN_MLE(X = X.stand[active.set.rows, active.set.columns,class==2 | class==3], class=ifelse(class[which(class==2 | class==3)]==2, 1, 2))
			U.mle = out$U
			V.mle = out$V

			## first need to do the correct screening
			mean.array.mle = array(0, dim=c(r, p, 2))
			for(jj in 1:r){
				for(kk in 1:p){
					if(out23.thresh[jj,kk]==0){
					mean.array.mle[jj,kk,1] <- mean.array.mle[jj,kk,2] <- mean(X.stand[jj,kk,class==2 | class==3])
					} else {
						mean.array.mle[jj,kk,1] <- mean(X.stand[jj,kk,class==2])
						mean.array.mle[jj,kk,2] <- mean(X.stand[jj,kk,class==3])
					}
				}
			}
			mean.array.mle = mean.array.mle[active.set.rows, active.set.columns, ]
			val23[k]=MN_CLASSIFICATION(X=X_val.stand[active.set.rows,active.set.columns, which(class_val==2 | class_val==3)], class=inner.class, pi.list = pihat, M = mean.array.mle, U = U.mle, V= V.mle)$misclass
		} else {

			active.set = which(out23.thresh==1)
			mean.vec.as = mean.vec[active.set, c(2,3)]
			vecX.c.as = vecX.c[,active.set]
			SigmaInv = solve(N^(-1)*crossprod(vecX.c.as))
			val23[k]= NORM_CLASSIFICATION_LDA(X=vecX_val[active.set, which(class_val==2 | class_val==3)], class=inner.class, pihat = pihat, Mean = mean.vec.as, SigmaInv = SigmaInv)$misclass
		
		}
	
	}

	# ------- store for later use
	ind.min23 = min(which(val23 == min(val23)))

# ------------------------------------------------
#  Fit full models + community vote 
# ------------------------------------------------

# ----  1-2 

		inner.class = ifelse(class[which(class==1 | class==2)]==1, 1, 2)
		out12 = abs(mean.array[,,1] - mean.array[,,2])
		out12.thresh = matrix(0, nrow=r, ncol=p)
		
		for(j in 1:ind.min12){
			check = which(out12 == max(out12[out12!=0]), arr.ind=T)
			out12.thresh[check[1], check[2]] <- 1 	
			out12[check[1], check[2]] <- 0
		 }
		
		 active.set.rows = which(rowSums(out12.thresh)!=0)
		 active.set.columns = which(colSums(out12.thresh)!=0)

		 if(length(active.set.rows)>1 & length(active.set.columns)>1){

		 	out12.thresh[active.set.rows, active.set.columns] <-1
		 	active.set.12 = which(out12.thresh==1)
		 	out = MN_MLE(X = X.stand[active.set.rows, active.set.columns, class==1 | class==2], class=inner.class)
			U.mle = out$U
			V.mle = out$V
			mean.array.mle = array(0, dim=c(r, p, 2))
			for(jj in 1:r){
				for(kk in 1:p){
					if(out23.thresh[jj,kk]==0){
						mean.array.mle[jj,kk,1] <- mean.array.mle[jj,kk,2] <- mean(X.stand[jj,kk, which(class==1 | class==2)])
					} else {
						mean.array.mle[jj,kk,1] <- mean(X.stand[jj,kk,class==1])
						mean.array.mle[jj,kk,2] <- mean(X.stand[jj,kk,class==2])
					}
				}
			}
			mean.array.mle = mean.array.mle[active.set.rows, active.set.columns, ]
			beta12 = c(U.mle%*%(mean.array.mle[,,1] - mean.array.mle[,,2])%*%V.mle)

		} else {

			active.set.12 = which(out12.thresh==1)
			mean.vec.as = mean.vec[active.set.12, ]
			mean.vec.diff12 = mean.vec.as[,1] - mean.vec.as[,2]
			vecX.c.as = vecX.c[,active.set.12]
			beta12 = solve(N^(-1)*crossprod(vecX.c.as), mean.vec.diff12)
		
		}

# ---- 1-3
			
			inner.class = ifelse(class[which(class==1 | class==3)]==1, 1, 2)
			out13 = abs(mean.array[,,1] - mean.array[,,3])
			out13.thresh = matrix(0, nrow=r, ncol=p)
			
			for(j in 1:ind.min13){
				check = which(out13 == max(out13[out13!=0]), arr.ind=T)
				out13.thresh[check[1], check[2]] <- 1 	
				out13[check[1], check[2]] <- 0
			 }
			 
			 active.set.rows = which(rowSums(out13.thresh)!=0)
			 active.set.columns = which(colSums(out13.thresh)!=0)

			 if(length(active.set.rows)>1 & length(active.set.columns)>1){
			 	
			 	out13.thresh[active.set.rows, active.set.columns] <-1
			 	active.set.13 = which(out13.thresh==1)
			 	out = MN_MLE(X = X.stand[active.set.rows, active.set.columns, class==1 | class==3], class=inner.class)
				U.mle = out$U
				V.mle = out$V
				mean.array.mle = array(0, dim=c(r, p, 2))
				for(jj in 1:r){
					for(kk in 1:p){
						if(out23.thresh[jj,kk]==0){
						mean.array.mle[jj,kk,1] <- mean.array.mle[jj,kk,2] <- mean(X.stand[jj,kk,which(class==1 | class==3)])
						} else {
							mean.array.mle[jj,kk,1] <- mean(X.stand[jj,kk,class==1])
							mean.array.mle[jj,kk,2] <- mean(X.stand[jj,kk,class==3])
						}
					}
				}
				mean.array.mle = mean.array.mle[active.set.rows, active.set.columns, ]
				beta13 = c(U.mle%*%(mean.array.mle[,,1] - mean.array.mle[,,2])%*%V.mle)

			} else {

				active.set.13 = which(out13.thresh==1)
				mean.vec.as = mean.vec[active.set.13, ]
				mean.vec.diff13 = mean.vec.as[,1] - mean.vec.as[,3]
				vecX.c.as = vecX.c[,active.set.13]
				beta13 = solve(N^(-1)*crossprod(vecX.c.as), mean.vec.diff13)

			}

# ----- 2-3

	inner.class = ifelse(class[which(class==2 | class==3)]==2, 1, 2)
	out23 = abs(mean.array[,,2] - mean.array[,,3])
	out23.thresh = matrix(0, nrow=r, ncol=p)
	
	for(j in 1:ind.min23){
		check = which(out23 == max(out23[out23!=0]), arr.ind=T)
		out23.thresh[check[1], check[2]] <- 1 	
		out23[check[1], check[2]] <- 0
	 }
	
	active.set.rows = which(rowSums(out23.thresh)!=0)
	active.set.columns = which(colSums(out23.thresh)!=0)

	if(length(active.set.rows)>1 & length(active.set.columns)>1){
		
		out23.thresh[active.set.rows, active.set.columns] <-1
		active.set.23 = which(out23.thresh==1)
		
		out = MN_MLE(X = X.stand[active.set.rows, active.set.columns, class==2 | class==3], class=inner.class)
		U.mle = out$U
		V.mle = out$V

		mean.array.mle = array(0, dim=c(r, p, 2))
		for(jj in 1:r){
			for(kk in 1:p){
				if(out23.thresh[jj,kk]==0){
					mean.array.mle[jj,kk,1] <- mean.array.mle[jj,kk,2] <- mean(X.stand[jj,kk,which(class==2 | class==3)])
				} else {
					mean.array.mle[jj,kk,1] <- mean(X.stand[jj,kk,class==2])
					mean.array.mle[jj,kk,2] <- mean(X.stand[jj,kk,class==3])
				}
			}
		}
		mean.array.mle = mean.array.mle[active.set.rows, active.set.columns, ]
		beta23 = c(U.mle%*%(mean.array.mle[,,1] - mean.array.mle[,,2])%*%V.mle)

	} else {

		active.set.23 = which(out23.thresh==1)
		mean.vec.as = mean.vec[active.set.23, ]
		mean.vec.diff23 = mean.vec.as[,2] - mean.vec.as[,3]
		vecX.c.as = vecX.c[,active.set.23]
		Sigma = (N)^(-1)*crossprod(vecX.c.as)
		beta23  = solve(Sigma, mean.vec.diff23)

	}

# ------------------------------------------
# Community vote
# ------------------------------------------	

	test.length = dim(vecX_test)[2]
	test.votes = matrix(0, nrow=test.length, ncol=3)
	class.rate = rep(0, test.length)

# ------- 1-2
	
	mean.array.mle = array(0, dim=c(r, p, 2))
	for(jj in 1:r){
		for(kk in 1:p){
			if(out12.thresh[jj,kk]==0){
				mean.array.mle[jj,kk,1] <- mean.array.mle[jj,kk,2] <- mean(X.stand[jj,kk,which(class==1 | class==2)])
			} else {
				mean.array.mle[jj,kk,1] <- mean(X.stand[jj,kk,class==1])
				mean.array.mle[jj,kk,2] <- mean(X.stand[jj,kk,class==2])
			}
		}
	}

	mean.array.vec = apply(mean.array.mle, 3, as.vector)
	for(j in 1:test.length){

		test12 = crossprod((vecX_test[active.set.12, j] - c(mean.array.vec[active.set.12, 1] + mean.array.vec[active.set.12, 2])/2), beta12)
		
		if(test12 > 0){
			test.votes[j, 1] <- test.votes[j, 1] + 1
		} else{
			test.votes[j, 2] <- test.votes[j, 2] + 1
		}

	}

# ------ 1-3
	mean.array.mle = array(0, dim=c(r, p, 2))
	for(jj in 1:r){
		for(kk in 1:p){
			if(out13.thresh[jj,kk]==0){
			mean.array.mle[jj,kk,1] <- mean.array.mle[jj,kk,2] <- mean(X.stand[jj,kk,which(class==1 | class==3)])
			} else {
				mean.array.mle[jj,kk,1] <- mean(X.stand[jj,kk,class==1])
				mean.array.mle[jj,kk,2] <- mean(X.stand[jj,kk,class==3])
			}
		}
	}

	mean.array.vec = apply(mean.array.mle, 3, as.vector)
	for(j in 1:test.length){
		test13 = crossprod((vecX_test[active.set.13, j] - c(mean.array.vec[active.set.13, 1] + mean.array.vec[active.set.13, 2])/2), beta13)
		
		if(test13 > 0){
			test.votes[j, 1] <- test.votes[j, 1]+ 1
		} else{
		 	test.votes[j, 3] <- test.votes[j, 3] + 1
		}

	}

# ------ 2-3
	mean.array.mle = array(0, dim=c(r, p, 2))
	for(jj in 1:r){
		for(kk in 1:p){
			if(out23.thresh[jj,kk]==0){
			mean.array.mle[jj,kk,1] <- mean.array.mle[jj,kk,2] <- mean(X.stand[jj,kk, which(class==2 | class==3)])
			} else {
				mean.array.mle[jj,kk,1] <- mean(X.stand[jj,kk,class==2])
				mean.array.mle[jj,kk,2] <- mean(X.stand[jj,kk,class==3])
			}
		}
	}
				
	mean.array.vec = apply(mean.array.mle, 3, as.vector)
		
	for(j in 1:test.length){
		test23 = crossprod((vecX_test[active.set.23, j] - c(mean.array.vec[active.set.23, 1] + mean.array.vec[active.set.23, 2])/2), beta23)
		
		if(test23 > 0){
			test.votes[j, 2] <- test.votes[j, 2] + 1
		} else{
			test.votes[j, 3] <- test.votes[j, 3] + 1
		}

	}

# ----- Clean output 
	for(j in 1:test.length){
		class.rate[j] = which.max(test.votes[j,]) != class_test[j]
	}
		
	tuned.MN.misclass.rate = mean(class.rate)

	## retun tuned_mean_array: 
	G12 = rep(0, r*p)
	G12[active.set.12] = 1

	G13 = rep(0, r*p)
	G13[active.set.13] = 1

	G23 = rep(0, r*p)
	G23[active.set.23] = 1


	G.MN.array.tuned = array(0, dim=c(r, p, 2, 2))
	G.MN.array.tuned[,,1, 1] = matrix(G12, byrow=FALSE, nrow=r, ncol=p)
	G.MN.array.tuned[,,1, 2] = matrix(G13, byrow=FALSE, nrow=r, ncol=p)
	G.MN.array.tuned[,,2, 2] = matrix(G23, byrow=FALSE, nrow=r, ncol=p)


	return(list("tuned.misclass.rate"=tuned.misclass.rate, "G.array.tuned" = G.array.tuned, 
	"tuned.MN.misclass.rate" = tuned.MN.misclass.rate, "G.array.tuned.MN" = G.MN.array.tuned
	))

}


require(compiler)
SURE_SCREEN <- cmpfun(SURE_SCREEN)
