
# ------ load packages
packgs="library(plyr);
library(glasso);
library(Rcpp);
library(RcppArmadillo);
library(compiler);\n"

# ------- source functions from folder
sources= "source(\"Functions/SAMPLES.R\");
source(\"Functions/GENERAL_ITER.R\");
source(\"Functions/MN_CLASSIFICATION.R\");
source(\"Functions/MatLDA.R\");
source(\"Functions/MatLDA_Grid.R\");
source(\"Functions/MN_MLE.R\");
source(\"Functions/GUO_L1_NB.R\");
source(\"Functions/SURE_SCREEN.R\");
source(\"Functions/VS_CHECK.R\");
source(\"Functions/NORM_CLASSIFICATION_LDA.R\"); 
source(\"Functions/CREATE_DAT_COV1.R\");
source(\"Functions/KRON_XU_GRID.R\"); 
source(\"Functions/ORACLE_MEAN_GRID.R\"); 
sourceCpp(\"Functions/M_Update_Rcpp.cpp\"); \n"

# ----- source simulation script
runsim="\nsource(\"Model1_Main.R\")"


N = 75; N_test = 1000; N_validate = 75
reps = c(100, 100, 100, 100, 100)
seed.in = c(1, 1, 1, 1, 1)
C = 3 
r = c(8, 16, 32, 32, 32)
p = c(8, 16, 32, 16, 8)
lam.guo = list(c(-3, 5),c(-3, 6), c(-1, 5), c(-2, 5),c(1, 5))
lam.Xu = list(c(-8, 0),c(-8, 0),c(-8, 0), c(-6, 2), c(-6, 2))
lam1.prop = list(c(-6, 0),c(-5, 0),c(-4, 0),c(-4, 0), c(-4, 0))
lam2.prop = list(c(-12, -4),c(-11, -4),c(-8, -3),c(-8, -1),c(-8, -2))
lam.mu = list(c(-10, 0),c(-11, 0), c(-8, 3),c(-8, 2),c(-8, 2))


for(i in 1:length(r)){
  
	filename=paste("Model1_r", r[i], "p", p[i],  sep="")
	savename=paste("Model1_r", r[i], "p", p[i], ".RDS", sep="")
	settings=paste("N = ", N, "; N_test = ", N_test, ";N_validate = ", N_validate, ";reps = ", reps[i], "; 
	seed = ", seed.in[i], "; r = ", r[i], "; p = ", p[i], "; C = ", C, ";\n 
	lambda.guo = c(2^seq(",lam.guo[[i]][1],",",lam.guo[[i]][2],", by=0.5));
	lambda1.Xu = c(2^seq(",lam.Xu[[i]][1],",",lam.Xu[[i]][2],", by=0.5));
	lambda1.prop = c(2^seq(",lam1.prop[[i]][1],",",lam1.prop[[i]][2],", by=0.5));
	lambda2.prop = c(2^seq(",lam2.prop[[i]][1],",",lam2.prop[[i]][2],", by=0.5));
	lambda2.mu = c(2^seq(",lam.mu[[i]][1],",",lam.mu[[i]][2],", by=0.5));\n 
	trans=FALSE;\n
	savename=\"", savename, "\";\n", sep="")
	write(x=paste(packgs, sources, settings, runsim, sep=""), file=paste(filename, ".R", sep=""))
	# command=paste("nohup R --no-save < ",filename, ".R > out_", i, filename, ".txt &", sep="") 
	# system(command)
 
}



# ------- set settings for simulations (r,p = 64,64)
N = 75; N_test = 1000; N_validate = 75
reps = rep(1, 100)
seed.in = 1:100
C = 3 
r = rep(64, 100)
p = rep(64, 100)
lam.guo = rep(list(c(1, 6)),100)
lam.Xu = rep(list(c(-4, 1)),100)
lam1.prop = rep(list(c(-4, -1.5)),100)
lam2.prop = rep(list(c(-8, -2.5)), 100)
lam.mu = rep(list(c(-9, -3)), 100)

for(i in 1:length(r)){
  
	filename=paste("Model1_r", r[i], "p", p[i], "_",i, sep="")
	savename=paste("Model1_r", r[i], "p", p[i], "_",i,".RDS", sep="")
	settings=paste("N = ", N, "; N_test = ", N_test, ";N_validate = ", N_validate, ";reps = ", reps[i], "; 
	seed = ", seed.in[i], "; r = ", r[i], "; p = ", p[i], "; C = ", C, ";
	lambda.guo = c(2^seq(",lam.guo[[i]][1],",",lam.guo[[i]][2],", by=0.5));
	lambda1.Xu = c(2^seq(",lam.Xu[[i]][1],",",lam.Xu[[i]][2],", by=0.5));
	lambda1.prop = c(2^seq(",lam1.prop[[i]][1],",",lam1.prop[[i]][2],", by=0.5)); 
	lambda2.prop = c(2^seq(",lam2.prop[[i]][1],",",lam2.prop[[i]][2],", by=0.5));
	lambda2.mu = c(2^seq(",lam.mu[[i]][1],",",lam.mu[[i]][2],", by=0.5));\n 
	trans = FALSE;\n
	savename=\"", savename, "\";\n", sep="")
	write(x=paste(packgs, sources, settings, runsim, sep=""), file=paste(filename, ".R", sep=""))
	# command=paste("nohup R --no-save < ",filename, ".R > out_", i, filename, ".txt &", sep="") 
	# system(command)
 
}




N = 75; N_test = 1000; N_validate = 75
reps = rep(1, 200)
seed.in = c(1:100, 1:100)
r = c(rep(64,100), rep(126, 100))
p = c(rep(32,100), rep(32, 100))
C = 3 
lam.guo = rep(list(c(-1, 5)), 200)
lam.Xu = append(rep(list(c(-6, 2)),100), rep(list(c(-4, 2)),100))
lam1.prop = rep(list(c(-4, 1.5)), 200)
lam2.prop = rep(list(c(-8, -2.5)),200)
lam.mu = append(rep(list(c(-8, -4)),100), rep(list(c(-8, -2)),100))

for(i in 1:length(r)){
	  
	if(i>100){
		filename=paste("Model1_r", p[i], "p", r[i], "_",i-100, sep="")
		savename=paste("Model1_r", p[i], "p", r[i], "_",i-100,".RDS", sep="")
	} else {
		filename=paste("Model1_r", p[i], "p", r[i], "_", i, sep="")
		savename=paste("Model1_r", p[i], "p", r[i], "_", i,".RDS", sep="")
	}

	settings=paste("N = ", N, "; N_test = ", N_test, ";N_validate = ", N_validate, ";reps = ", reps[i], "; 
	seed = ", seed.in[i], "; r = ", r[i], "; p = ", p[i], "; C = ", C, ";\n 
	lambda.guo = c(2^seq(",lam.guo[[i]][1],",",lam.guo[[i]][2],", by=0.5));
	lambda1.Xu = c(2^seq(",lam.Xu[[i]][1],",",lam.Xu[[i]][2],", by=0.5));
	lambda1.prop = c(2^seq(",lam1.prop[[i]][1],",",lam1.prop[[i]][2],", by=0.5));
	lambda2.prop = c(2^seq(",lam2.prop[[i]][1],",",lam2.prop[[i]][2],", by=0.5));
	lambda2.mu = c(2^seq(",lam.mu[[i]][1],",",lam.mu[[i]][2],", by=0.5));\n 
	trans = TRUE;\n 
	savename=\"", savename, "\";\n", sep="")
	write(x=paste(packgs, sources, settings, runsim, sep=""), file=paste(filename, ".R", sep=""))
	# command=paste("nohup R --no-save < ",filename, ".R > out_", i, filename, ".txt &", sep="") 
	# system(command)
 
}






