library(ggplot2)

# --- readin Zhong model results  (grey boxplots)
load("Model_ZS_r3_p36_Model5.RData")
results = c(mda.mc, pmda.cont.mc, MNlda.mc, pmn.mc, sda2.mc, lda.mc, rda.mc, lasso.mc)
labs = c(rep("MDA", 100), rep("PMDA", 100), rep("MN", 100), rep("PMN", 100), rep("Clemmensen", 100), rep("LDA", 100), rep("RDA", 100), rep("Lasso", 100)) 
Fullr6 = data.frame("Misclassification" = results, "Methods" = labs, r = rep("Model 5", 800))

# --- readin modified Zhong model results (white boxplots)
load("Model_ZS_r3_p36_Model6.RData")
results = c(mda.mc, pmda.cont.mc, MNlda.mc, pmn.mc, sda2.mc, lda.mc, rda.mc, lasso.mc)
labs = c(rep("MDA", 100), rep("PMDA", 100), rep("MN", 100), rep("PMN", 100), rep("Clemmensen", 100), rep("LDA", 100), rep("RDA", 100), rep("Lasso", 100)) 
Fullr3 = data.frame("Misclassification" = results, "Methods" = labs, r = rep("Model 6", 800))

# --- shape data for ggplot  --------
stack = do.call("rbind", list(Fullr3, Fullr6))
stack$Methods = factor(stack$Methods, levels=c("MDA", "PMDA", "MN", "PMN", "Clemmensen", "LDA", "RDA", "Lasso"), ordered=TRUE)
stack$r = factor(stack$r, levels=c("Model 5", "Model 6"), ordered=TRUE)

# ---- make plot ---------------
p = ggplot(aes(y = Misclassification, x = Methods, fill=r), data = stack) + geom_boxplot() + 
  scale_fill_manual(values = c("grey", "white"), guide=FALSE) + theme_bw() + ylab("Misclassification rate") + theme(legend.title=element_blank(), axis.text.x = element_text(angle = 15, vjust = 0.5, hjust = 0.5))
dev.new(width=5.5, height=3.75)
p
