library(plyr)
library(glasso)
library(Rcpp)
library(RcppArmadillo)
library(compiler)
library(sparseLDA)
library(dummies)
library(klaR)
library(msda)
library(MASS)
library(glmnet)

source("~/Functions/SAMPLES.R") 
source("~/Functions/GENERAL_ITER.R")
source("~/Functions/MN_CLASSIFICATION.R") 
source("~/Functions/MatLDA.R")
source("~/Functions/MatLDA_Grid.R") 
source("~/Functions/MN_MLE.R")
source("~/Functions/GUO_L1_NB.R") 
source("~/Functions/SURE_SCREEN.R")
source("~/Functions/NORM_CLASSIFICATION_LDA.R")
sourceCpp("~/Functions/M_Update_Rcpp.cpp") 
source("~/Functions/CREATE_DAT_ZS.R") 
source("~/Functions/PMCA.R")


# --- set number of replications
reps = 100

# --- allocate memory
rda.mc = rep(0, reps)
mda1.mc = rep(0, reps)
pmda1.mc = rep(0, reps)
mda2.mc = rep(0, reps)
pmda2.mc = rep(0, reps)
pmda3.mc = rep(0, reps)
pmda4.mc = rep(0, reps)
mda.mc = rep(0, reps)
pmda.cont.mc = rep(0, reps)
pmda.disc.mc = rep(0, reps)
MNlda.mc = rep(0, reps)
pmn.mc = rep(0, reps)
pmn.time = rep(0, reps)
pmda1.time = rep(0, reps)
pmda2.time = rep(0, reps)
lda.mc = rep(0, reps)
mda.mc = rep(0, reps)
sda1.mc = rep(0, reps)
sda2.mc = rep(0, reps)
lasso.mc = rep(0, reps)


# ------ set dimensionality 
N = 120
N_test = 500
N_validate = 120
C = 2
r = 3
p = 36

# ------ set seed for reproducibility
set.seed(100)

for(uu in 1:reps){

	# ---  create data 
	dat1 = CREATE_DAT_ZS_MOD(r=3, p=36, C=2, N, N_validate, N_test)

	# --- rename created data
	X = dat1$X
	class = dat1$class
	X_val = dat1$X_val
	X_test = dat1$X_test

	# -- population mean
	class_test = dat1$class_test
	class_val = dat1$class_val
	nc = count(class)[,2]
	pihat = nc/N
	vecX_test = apply(X_test, 3, as.vector)
	vecX_val = apply(X_val, 3, as.vector)

	# ---- transpose data for use in Zhong and Suslick code
	Xt = array(0, dim=c(p, r, length(class)))
 	for(lll in 1:length(class)){
 		Xt[,,lll] = t(dat1$X[,,lll])
 	}

 	Xt_val = array(0, dim=c(p, r, length(class_val)))
 	for(lll in 1:length(class_val)){
 		Xt_val[,,lll] = t(dat1$X_val[,,lll])
 	}

 	Xt_test = array(0, dim=c(p, r, length(class_test)))
 	for(lll in 1:length(class_test)){
 		Xt_test[,,lll] = t(dat1$X_test[,,lll])
 	}

 	vecXt_test = apply(Xt_test, 3, as.vector)
 	vecXt_val = apply(Xt_val, 3, as.vector)

	# -------- Clemmensen
	vecX = t(apply(X,3, as.vector))
	Y = matrix(0, ncol=2, nrow=dim(vecX)[1])
	Y[,1] = ifelse(class==1, 1, 0) 
	Y[,2] = ifelse(Y[,1]==1, 0, 1)
	colnames(Y) = c("1", "2")

	# --- Normalize data
	Xc = normalize(vecX)
	Xn = Xc$Xc
	Xval_vec_norm = normalizetest(t(apply(X_val,3, as.vector)),Xc)
	Xtest_vec_norm = normalizetest(t(apply(X_test,3, as.vector)),Xc)

	# ---- Perform SDA with one non-zero loading for each discriminative
	stop.param = 2^seq(-8, 2, by=.25)
	sda.val = matrix(0, nrow=length(stop.param), ncol=1)
	sda.test = matrix(0, nrow=length(stop.param), ncol=1)

	for(k in 1:length(stop.param)){
		out <- sda(Xn, Y, stop = stop.param[k], maxIte = 25, trace = FALSE)
		sda.val[k,1] <- sum(predict(out, Xval_vec_norm)$class != class_val)/length(class_val) 
		sda.test[k,1] <- sum(predict(out, Xtest_vec_norm)$class != class_test)/length(class_test)
	}

	ind2 = min(which(sda.val == min(sda.val), arr.ind=TRUE)[,2])
  	ind1 = max(which(sda.val[,ind2] == min(sda.val[,ind2])))
	sda2.mc[uu] = sda.test[ind1, ind2]

	# ----- RDA -----------------
	gamma = seq(0.005, .995, length=15)
	lambda = seq(0.005, 0.995, length=15)
	rda.val = matrix(0, nrow=length(gamma), ncol=length(lambda))
	rda.test = matrix(0, nrow=length(gamma), ncol=length(lambda))
	for(j in 1:length(gamma)){
		for(k in 1:length(lambda)){
			out.rda <- rda(x= t(apply(X, 3, as.vector)), grouping=class,  gamma = gamma[j], lambda = lambda[k])
			rda.val[j,k] = sum(predict(out.rda, newdata = data.frame(t(vecX_val)))$class != class_val)/length(class_val)
			rda.test[j,k] = sum(predict(out.rda, newdata = data.frame(t(vecX_test)))$class != class_test)/length(class_test)
		}
	}
	rda.mc[uu] = rda.test[which(rda.val==min(rda.val), arr.ind=TRUE)[sum(rda.val==min(rda.val)), ][1],which(rda.val==min(rda.val), arr.ind=TRUE)[sum(rda.val==min(rda.val)), ][2]]

	# ---- LDA --------------------
 	out = lda(x = t(apply(X, 3, as.vector)), grouping = class)
	lda.mc[uu] = sum(predict(out, newdata = t(vecX_test))$class != class_test)/length(class_test)

	# ----------------------------------------------------
	# PMDA & MDA: Continuous tuning  (pmda elastic net penalty parameter = 50)
	# ----------------------------------------------------
	init = runif(r)
	init = init/sqrt(crossprod(init))
	out.check = PMA(t(apply(X, 3, as.vector)),y=class,K=2,p=p,q=r,d=1, lambda = 0, xao=init)
	mda1.mc[uu] = sum(predict.PMA(out.check,newdata=t(vecX_test),p=p,q=r,d=1)!=class_test)/length(class_test)
	mda1.val =  sum(predict.PMA(out.check,newdata=t(vecX_val),p=p,q=r,d=1)!=class_val)/length(class_val)

	lambda = c(2^seq(-6, 6, by=.25))
	pmda1.val = rep(0, length(lambda))
	pmda1.test = rep(0, length(lambda))
	ptm = proc.time()
	for(j in 1:length(lambda)){
		out.check = PMA(t(apply(X, 3, as.vector)),y=class,K=2,p=p,q=r,d=1, lambda = lambda[j], xao=init)
		pmda1.val[j] = sum(predict.SPMA(out.check,newdata=t(vecX_val),p=p,q=r,d=1)!=class_val)/length(class_val)
		pmda1.test[j]= sum(predict.SPMA(out.check,newdata=t(vecX_test),p=p,q=r,d=1)!=class_test)/length(class_test)
	}

	# ----- smallest model that minimizes misclassification error on validation set 
	pmda1.mc[uu] = pmda1.test[max(which(pmda1.val == min(pmda1.val)))]
	pmda1.time[uu] = (proc.time() - ptm)[3]

	# ----- Oracle starting values 
	out.check = PMA(t(apply(X, 3, as.vector)),y=class,K=2,p=p,q=r,d=1, lambda = 0, xao=c(1/sqrt(2),1/sqrt(2),rep(0, r-2)))
	mda2.mc[uu] = sum(predict.PMA(out.check,newdata=t(vecX_test),p=p,q=r,d=1)!=class_test)/length(class_test)
	mda2.val =  sum(predict.PMA(out.check,newdata=t(vecX_val),p=p,q=r,d=1)!=class_val)/length(class_val)
	
	lambda = c(2^seq(-6, 6, by=.25))
	pmda2.val = rep(0, length(lambda))
	pmda2.test = rep(0, length(lambda))
	ptm + proc.time()
	for(j in 1:length(lambda)){
		out.check = PMA(t(apply(X, 3, as.vector)),y=class,K=2,p=p,q=r,d=1, lambda = lambda[j], xao=c(1/sqrt(2),1/sqrt(2),rep(0, r-2)))
		pmda2.val[j] = sum(predict.SPMA(out.check,newdata=t(vecX_val),p=p,q=r,d=1)!=class_val)/length(class_val)
		pmda2.test[j]= sum(predict.SPMA(out.check,newdata=t(vecX_test),p=p,q=r,d=1)!=class_test)/length(class_test)
	}

	pmda2.mc[uu] = pmda2.test[max(which(pmda2.val == min(pmda2.val)))]
	pmda2.time[uu] = (proc.time() - ptm)[3]
	

	# --- select initial values based on validation set -- if same, use oracle start values 
	if(min(pmda2.val) <= min(pmda1.val)){
		pmda.cont.mc[uu] = pmda2.mc[uu] 
	} else {
		pmda.cont.mc[uu] = pmda1.mc[uu]
	}

	if(mda2.val <= mda1.val){
		mda.mc[uu] = mda2.mc[uu]
	} else {
		mda.mc[uu] = mda1.mc[uu]
	}

	# ---------------------------------------------------
	# PMDA: Discrete tuning (pmda elastic net penalty parameter = 50)
	# --------------------------------------------------
	init = runif(r)
	init = init/sqrt(crossprod(init))

	lambda = -36:-1
	pmda3.val = rep(0, length(lambda))
	pmda3.test = rep(0, length(lambda))

	for(j in 1:length(lambda)){
		out.check = PMA(t(apply(X, 3, as.vector)),y=class,K=2,p=p,q=r,d=1, lambda = lambda[j], xao=init)
		pmda3.val[j] = sum(predict.SPMA(out.check,newdata=t(vecX_val),p=p,q=r,d=1)!=class_val)/length(class_val)
		pmda3.test[j]= sum(predict.SPMA(out.check,newdata=t(vecX_test),p=p,q=r,d=1)!=class_test)/length(class_test)
	}

	pmda3.mc[uu] = pmda3.test[max(which(pmda3.val == min(pmda3.val)))]

	lambda = -36:-1
	pmda4.val = rep(0, length(lambda))
	pmda4.test = rep(0, length(lambda))

	for(j in 1:length(lambda)){
		out.check = PMA(t(apply(X, 3, as.vector)),y=class,K=2,p=p,q=r,d=1, lambda = lambda[j], xao=c(1/sqrt(2),1/sqrt(2),rep(0, r-2)))
		pmda4.val[j] = sum(predict.SPMA(out.check,newdata=t(vecX_val),p=p,q=r,d=1)!=class_val)/length(class_val)
		pmda4.test[j]= sum(predict.SPMA(out.check,newdata=t(vecX_test),p=p,q=r,d=1)!=class_test)/length(class_test)
	}

	pmda4.mc[uu] = pmda4.test[max(which(pmda4.val == min(pmda4.val)))]

	
	if(min(pmda4.val)<=min(pmda3.val)){
		pmda.disc.mc[uu] = pmda4.mc[uu]
	} else {
		pmda.disc.mc[uu] = pmda3.mc[uu]
	}

	# -------- Matrix-Normal MLE 
	outMLE = MN_MLE(X=Xt, class)
	result = MN_CLASSIFICATION(Xt_test, class_test, 
		outMLE$Mean, outMLE$U,  outMLE$V, pihat)
	MNlda.mc[uu] = result$misclass
	

	# ---------- PMN; our proposal 
	lambda1 = c(2^seq(-12, -4, by=.25))
	lambda2 = c(2^seq(-8, 2, by=.25))
	
	ptm = proc.time()
	fit.grid = MatLDA_Grid(X=Xt, class=class, lambda1=lambda1, 
		lambda2=lambda2, quiet=TRUE,
		Xval=Xt_val, classval= class_val,
		k.iter = 100, cov.tol=1e-5, m.tol=1e-5, full.tol=1e-6)

	CV.mat = fit.grid$Val.mat
	G.mat = fit.grid$G.mat*(CV.mat==min(CV.mat))
	ind1 = (which(G.mat==max(G.mat), arr.ind=TRUE))[,2]
	ind2 = (which(G.mat==max(G.mat), arr.ind=TRUE))[,1]
	out = unique(ind2[which(ind2==max(ind2))])
	lambda1.cv = lambda1[out]
	out2 = unique(max(ind1[ind2==out]))
	lambda2.cv = lambda2[out2]

	out = MatLDA(X=Xt, class=class, lambda1=lambda1.cv, 
		lambda2=lambda2.cv, quiet=TRUE,
		Xval=NULL, classval= NULL,
		k.iter = 100, cov.tol=1e-5, m.tol=1e-5, full.tol=1e-6)

	result = PredictMN(out, newdata=Xt_test, newclass=class_test)
	pmn.mc[uu] = result$misclass
	pmn.time[uu] = (proc.time() - ptm)[3]
	
	# --- L1-penalized logistic regression ----------- 
	out = glmnet(x = vecX, y =class, family="binomial")
	lambda = out$lambda
	check = colSums(predict(out, newx = t(vecX_val), type = "class", s =  out$lambda) != class_val)/length(class_val)
	optim.lambda = max(which(check==min(check)))
	lasso.mc[uu] = colSums(predict(out, newx = t(vecX_test), type = "class", s = lambda[optim.lambda]) != class_test)/length(class_test)

	save(sda1.mc, 
		sda2.mc,	
		rda.mc,
		mda1.mc,
		mda2.mc,
		pmda1.val,
		pmda2.val,
		pmda1.test,
		pmda2.test,
		mda.mc,
		pmda1.mc,
		pmda2.mc,
		pmda.cont.mc,
		pmda.disc.mc,
		pmda3.mc,
		pmda4.mc,
		MNlda.mc,
		pmn.time, 
		pmda1.time, 
		pmda2.time, 
		pmn.mc,
		lda.mc,
		mda.mc, 
		lasso.mc, 
		file="Model_ZS_r3_p36_Model6.RData")

}

unlink(".RData")